#!/usr/bin/lua

local json = require("luci.json")
local io = require("io")
local fs = require("nixio.fs")
local os = require("os")

local sys = require "luci.sys"
local utl = require "luci.util"
local uci = require "luci.model.uci".cursor()

CODE_SUCCESS                  = 0      -- // ɹ
CODE_NXP_SECURITY_HANDINGSHAKE= 96 	-- // NXP豸ʱ豸ѾͨѶϣڽмֲ
CODE_UPGRADE_STARTED          = 97     -- // ʼ
CODE_UPGRADE_SUCCESS          = 98     -- // ɹ
CODE_WAIT_TO_EXECUTE          = 99     -- // Ѿյȴִ
CODE_UNKNOW_ERROR             = 199    -- // δ֪
CODE_WRONG_FORMAT             = 101    -- // ĸʽ
CODE_UNKNOW_DEVICE            = 102    -- // δ֪豸ָ豸
CODE_UNKNOW_ATTRIBUTE         = 103    -- // δֵ֪
CODE_TIMEOUT                  = 104    -- // ʱ
CODE_BUSY                     = 105    -- // 豸æѾдִеͬʱ
CODE_UNKNDOW_CMD              = 106    -- // δ֪
CODE_NOT_SUPPORTED            = 107    -- // ֵ֧Ĳ
CODE_UPGRADE_MD5SUM_FAILED    = 108    -- // ʱMD5SUMУʧ
CODE_UPGRADE_DOWNLOAD_FAILED  = 109    -- // ʱ̼ʧ
CODE_UPGRADE_FAILED           = 110    -- // ʧ
CODE_PASSWORD_ALREADY_EXISTS  = 111    -- // Ѿ
CODE_PASSWORD_FULL            = 112    -- // Ѿ
CODE_PASSWORD_NOT_EXISTS      = 113    -- // Ҫɾ벻ɹ
CODE_MINUS_1                  = -1     -- // δִҲִеɷ˸

--: util function-------------------------------------------------------------------

function my_print(s)
	--os.execute('logger -s \'' .. os.time() .. ' [INFO] ChenBei::' .. s .. '\'')
	--print('logger -s \'' .. os.time() .. ' [INFO] ChenBei::' .. s .. '\'')
	log_debug(s)
end

function get_uuid() 
	local __uuid = luci.util.exec('cat /proc/sys/kernel/random/uuid | tr -d "\r\n-"')
	return __uuid
end

function uptime() 
	local file=io.open("/proc/uptime", "r")
	if (file == nil) then
		return "0"
	end
	local ut = file:read("*n")
	-- my_print(ut)
	file:close()
	return math.floor(ut)
end
function get_version() 
	local file=io.open("/etc/dusun_build", "r")
	if (file == nil) then
		return "unknown"
	end
	local ver = file:read("*l")
	file:close()
	-- my_print(ver)
	ver = string.gsub(ver, "%s+", "")
	ver = string.gsub(ver, "BUILD_VERSION=", "")
	return string.gsub(ver, "_%w+", "")
end
function get_model()
	local model=fs.readfile("/tmp/sysinfo/model")
	return  string.gsub(model, "%s+", "")
end
function get_factory()
	return "dusun"
end

function gwmac() 
	local mac=fs.readfile("/sys/class/net/eth0/address")
	return  string.gsub(mac, "%s+", "")
end

g_device_code	= ""
function get_devcode() 
	if (g_device_code ~= "") then
		return g_device_code
	end
	local file=io.open("/etc/dusun/device_code")
	if (file ~= nil) then
		g_device_code=fs.readfile("/etc/dusun/device_code")
		file:close()
	end
	return g_device_code
end

function set_devcode(deviceCode) 
	g_device_code = deviceCode
	fs.writefile("/etc/dusun/device_code", deviceCode)
end

function time() 
	return os.time()	
end

function time_ms() 
	local x,y = timeofday()
	return x * 1000 + y
end


_gateway_device_id = nil
function get_gateway_device_id()
	-- cat /proc/sys/kernel/random/uuid
	-- return luci.sys.exec('cat /proc/sys/kernel/random/uuid')
	my_print('gwmac is ' .. gwmac())
	if (_gateway_device_id == nil) then
		local t = io.popen('echo DusunGateway_' .. gwmac() .. ' | md5sum | xargs | cut -d " " -f 1')
		_gateway_device_id = 'ds' .. string.gsub(t:read('*all'), "%s+", "")
		t:close()
	end
	return _gateway_device_id
end
function get_gateway_model() 
	-- return luci.sys.exec('cat /tmp/sysinfo/model')
	-- return get_model()
	return 'hub_ds'
end
function get_gateway_version()
	-- return luci.sys.exec('cat /etc/dusun_build  | head -n1 | xargs | cut -d = -f 2')
	return get_version()
end
function get_gateway_battery() 
	return 100
end
function get_gateway_power_type()
	-- 0 battery, 1 outlet power
	return 1
end
function get_gateway_arm_mode() 
	-- 0 disarm, 1 arm home , 2 arm away
	-- return luci.sys.exec('[ -e /etc/config/dusun/mqtt-gw/bufang ] && cat /etc/config/dusun/mqtt-gw/bufang || echo 0')
	-- local t = io.popen('[ -e /etc/config/dusun/mqtt-gw/bufang ] && cat /etc/config/dusun/mqtt-gw/bufang || echo 0')
	local t = io.popen('[ -e /etc/config/dusun/amber/arm.db ] && cat /etc/config/dusun/amber/arm.db || echo 0')
	local x = t:read('*all')
	t:close()
	local rx = tonumber(string.sub(x, 1, 1))
	if (rx == 3) then
		rx = 2
	end
	return rx
end


function arm_mode_led_over_func(arg) 
		luci.util.exec('gpio l 62 4000 0 1 0 4000')
		luci.util.exec('gpio l 63 0 4000 0 1 4000')
end
function set_gateway_arm_mode(armMode) 
	-- return luci.sys.call('echo ' .. armMode .. ' > /etc/config/dusun/mqtt-gw/bufang')
	--local t = io.popen('mkdir -p /etc/config/dusun/mqtt-gw; echo ' .. armMode .. ' > /etc/config/dusun/mqtt-gw/bufang')
	if (armMode == 2) then
		armMode = 3
	end
	local t = io.popen('mkdir -p /etc/config/dusun/amber; echo ' .. armMode .. ' > /etc/config/dusun/amber/arm.db')
	t:close()

	if (armMode ==  0) then
		-- disabled
	elseif (armMode == 1) then
		-- arm home
		luci.util.exec('gpio l 62 10 10 1 0 3')
		luci.util.exec('gpio l 63 10 10 1 0 3')
		cache_schedule_add('arm_mode_led', 6, 'arm_mode_led_over_func', 0)
	else 
		-- arm away
		luci.util.exec('gpio l 62 10 10 1 0 3')
		luci.util.exec('gpio l 63 10 10 1 0 3')
		cache_schedule_add('arm_mode_led', 6, 'arm_mode_led_over_func', 0)
	end

	return 0
end
function get_gateway_sn() 
	-- mac without ':'
	-- return luci.sys.exec('cat /sys/class/net/eth0/address | sed "s/://g"')
	return gwmac()
end
function get_gateway_network_type() 
	-- 4G, wifi, eth
	-- return luci.sys.exec('[ -e /usr/bin/network_type.sh ] && /usr/bin/network_type.sh || echo eth')
	-- return 'eth'
	local ntype = string.gsub(luci.util.exec('[ -e /tmp/._network_type ] && cat /tmp/._network_type || echo eth'), "%s+", "")
	if (ntype == '4g') then
		ntype = '4G'
	end
	return ntype
end
function quanity2rssi(quanity) 
	if (quanity <= 0) then
		return -100
	elseif (quanity >= 100) then
		return -50
	else
		return math.floor((quanity/2) - 100)
	end
		
end
function get_gateway_network_strength() 
	-- a value, 
	-- return luci.sys.exec('[ -e /usr/bin/network_strength.sh ] && /usr/bin/network_strngth.sh || echo 100')
	-- return 100
	local rssi = string.gsub(luci.util.exec('[ -e /tmp/._network_strength ] && cat /tmp/._network_strength || echo 100'), "%s+", "")
	local iRssi = tonumber(rssi) or 0
	if (get_gateway_network_type() == 'wifi') then
		iRssi = quanity2rssi(tonumber(rssi))
	end
	
	if (get_gateway_network_type() == '4G') then
		if (iRssi == 0) then
			local rssi = string.gsub(luci.util.exec('[ -e /tmp/._csq ] && cat /tmp/._csq || echo 0'), "%s+", "")
			iRssi = tonumber(rssi) or 0
		end
	end

	return iRssi
end

function get_gateway_qccid()
	local qccid = string.gsub(luci.util.exec('[ -e /tmp/._qccid ] && cat /tmp/._qccid || echo 0'), "%s+", "")
	return qccid or ''
end

function get_gateway_rsrp() 
	local _rsrp = string.gsub(luci.util.exec('[ -e /tmp/._rsrp ] && cat /tmp/._rsrp || echo 0'), "%s+", "")
	local rsrp = tonumber(_rsrp) or 0
	return rsrp
end
function get_gateway_snr()
	--local _snr = string.gsub(luci.util.exec('[ -e /tmp/._snr ] && cat /tmp/._snr || echo 0'), "%s+", "")
	local _snr = string.gsub(luci.util.exec('[ -e /tmp/._snr ] && cat /tmp/._snr'), "%s+", "")
	local snr = tonumber(_snr) 
	return snr
end
function get_gateway_qnwinfo() 
  --local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwinfo ] && cat /tmp/._qnwinfo || echo 0'), "%s+", "")
  local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwinfo ] && cat /tmp/._qnwinfo'), "%s+", "")
  --return qnwinfo  or ''
  return qnwinfo 
end

function get_gateway_qnwvendor() 
  local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwvendor ] && cat /tmp/._qnwvendor'), "%s+", "")
  return qnwinfo 
end

function get_gateway_network_id() 
	-- wifi -> ssid, 4g isp
	-- return luci.sys.exec('[ -e /usr/bin/network_id.sh ] && /usr/bin/network_id.sh || echo 0')
	-- return 'eth'
	
	local _nid = string.gsub(luci.util.exec('[ -e /tmp/._network_id ] && cat /tmp/._network_id || echo eth'), "%s+", "");
	if (get_gateway_network_type() == '4G') then
		_nid = get_gateway_qnwvendor()
	end
	return _nid
end

function get_gateway_qnwchan() 
  --local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwchan ] && cat /tmp/._qnwchan || echo 0'), "%s+", "")
  local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwchan ] && cat /tmp/._qnwchan'), "%s+", "")
  --return qnwinfo or ''
  return qnwinfo
end
function get_gateway_qnwact() 
  --local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwact ] && cat /tmp/._qnwact || echo 0'), "%s+", "")
  local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwact ] && cat /tmp/._qnwact'), "%s+", "")
  --return qnwinfo or ''
  return qnwinfo 
end
function get_gateway_qnwband() 
  --local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwband ] && cat /tmp/._qnwband || echo 0'), "%s+", "")
  local qnwinfo = string.gsub(luci.util.exec('[ -e /tmp/._qnwband ] && cat /tmp/._qnwband'), "%s+", "")
  --return qnwinfo or ''
  return qnwinfo
end





function get_gateway_uptime()
	return uptime()
end
	
function get_gateway_sta_ip() 
	local wifi_ip   = luci.util.exec('ifconfig apcli0 | grep "inet addr" | xargs | cut -d ":" -f 2 | cut -d " " -f 1 | tr -d "\r\n"')
	return wifi_ip
end
function get_gateway_dial_cnt()
  local _dialcnt = string.gsub(luci.util.exec('[ -e /tmp/._dialcnt ] && cat /tmp/._dialcnt || echo 0'), "%s+", "")
  local dialcnt = tonumber(_dialcnt)
  return dialcnt
end
function get_gateway_off_cnt()
  local _offcnt = string.gsub(luci.util.exec('[ -e /tmp/._offcnt ] && cat /tmp/._offcnt || echo 0'), "%s+", "")
  local offcnt = tonumber(_offcnt)
  return offcnt
end

--[[
{model = 'Gateway',			devstr = 'Gateway',			sleep = 0,	 fnl = {
	{ fn = 'RemoteSh',					tp = 's'}, 
}},
	{model = 'YRD216 PBDB', devstr = 'lock_actor_ds',			sleep = 0,	 fnl = {}},
	{model = 'DoWi',				devstr = 'door_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'IndEnv',			devstr = 'temp_humi_sensor_ds',	sleep = 0,	 fnl = {}},
--]]
--TODO:, modify the lock type 

dev_model_maps = {
	{model = 'Z3FireSensor',devstr = 'smoke_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'Leak',				devstr = 'water_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'DoWi',				devstr = 'entry_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'MoSe 02',			devstr = 'motion_sensor_ds',		sleep = 0,	 fnl = {}},
	{model = 'YRD216 PBDB', devstr = 'smart_lock_ds',				sleep = 0,	 fnl = {}},
	{model = 'ASL120',			devstr = 'asl120',				sleep = 0,	 fnl = {}},
	{model = 'asl120',			devstr = 'asl120',				sleep = 0,	 fnl = {}},

	{model = 'ZB-WaterSensor-D0000',			 devstr = 'awd220',	sleep = 0,	 fnl = {}},
	{model = 'ZB-DoorSensor-D0001',				 devstr = 'aes220',	sleep = 0,	 fnl = {}},
	{model = 'ZB-DWSensor-S0000001',				 devstr = 'aes220',	sleep = 0,	 fnl = {}},
	{model = 'ZB-MotionSensor-S00000001',	 devstr = 'ams220',	sleep = 0,	 fnl = {}},
	{model = 'ZB-KeypadGeneric-D0000',		 devstr = 'ask110',						sleep = 0,	 fnl = {}},

	{model = 'ZBT-OnOffPlug-D0001',   devstr = 'azr110',   sleep = 0,       fnl = {}},
	{model = 'SmartPlug',   devstr = 'azr110',   sleep = 0,       fnl = {}},
	{model = 'ZB-SirenSensor-D0000',  devstr = 'aha110',   sleep = 0,       fnl = {}},
	{model = 'FFZB1-SM-ECO',devstr = 'als110',			sleep = 0,	 fnl = {}},
}
--[[
{model = 'siren',		 devstr = 'ask110',						sleep = 0,	 fnl = {}},
{model = 'switch',		 devstr = 'ask110',						sleep = 0,	 fnl = {}},
--]]
function devstr_search_by_modelstr(modelstr) 
	if (modelstr == nil) then
		return 'UnknownDevice',0
	end
	

	-- my_print('num : ' .. #dev_model_maps)
	
	for i=1,#dev_model_maps do
		local x = dev_model_maps[i]
		local model=x['model']
		-- my_print('model is :' .. model .. ', modelstr is :' .. modelstr)
		if ( model == modelstr ) then
			return x.devstr, x.sleep
		end
	end

	return 'UnknownDevice',0
end

function devfnl_search_by_modelstr(modelstr) 
	if (modelstr == nil) then
		return {}
	end
	
	-- my_print('num : ' .. #dev_model_maps)
	for i=1,#dev_model_maps do
		local x = dev_model_maps[i]
		local model=x['model']
		-- my_print('model is :' .. model .. ', modelstr is :' .. modelstr)
		if ( model == modelstr ) then
			return x.fnl
		end
	end

	return {}
end

function backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data) 
	my_print('backup cmd..')
	local fdir = '/tmp/chenbei/' .. requestMethod .. '/' .. deviceId
	my_print('fdir is ' .. fdir)
	-- /tmp/chenbei/getDeviceInfo/dsb66d9c8e656edcaa1eb6776a5b923a5a
	-- os.execute('mkdir -p ' .. fdir)
	local xcmd = 'mkdir -p ' .. fdir
	my_print('xcmd is ' .. xcmd)
	local t = io.popen(xcmd)
	t:close()
	local s = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			context				= {
				traceId					= traceId,
				requestMethod		= requestMethod,
				requestTime			= requestTime,
				deviceId				= deviceId,
				deviceModel			= deviceModel,
			},
			data					= data
		}
	}
	if (requestMethod == 'addSubDevice') then
		luci.util.exec('rm -rf ' .. fdir .. '/*')
	end
	if (requestMethod == 'getDeviceInfo') then
		luci.util.exec('rm -rf ' .. fdir .. '/*')
	end
	if (requestMethod == 'setLockState') then
		luci.util.exec('rm -rf ' .. fdir .. '/*')
	end
	t = io.open(fdir .. '/' .. traceId, 'w')
	t:write(json.encode(s))
	t:close()
end

function fetch_one_cmd(requestMethod, deviceId, subDeviceId)
	my_print('fetch one cmd: ' .. requestMethod .. ',deviceId :' .. deviceId)

	local fdir = '/tmp/chenbei/' .. requestMethod .. '/' .. deviceId .. '/'
	local cmd='ls ' .. fdir  .. ' 2>/dev/null | xargs | cut -d " " -f 1'
	local t = io.popen(cmd)
	local y = t:read('*all')
	y = string.gsub(y, "%s+", "")
	t:close()

	if (string.len(y) > 0) then
		local f = fdir .. y
		local t = io.popen('cat ' .. f .. ' 2>/dev/null')
		local s = t:read('*all')
		t:close()
	
		t = io.popen('rm -rf ' .. f)
		t:close()
		my_print(' s is' .. s)

		local js = json.decode(s)
		if (js == nil or js['fixedHeader'] == nil) then
			my_print('nil fixedHeader')
			return nil, nil, nil
		end

		if (js['variableHeader'] == nil) then
			my_print('nil variableHeader')
			return nil, nil, nil
		end

		local payload = js['payload']
		if (payload == nil) then
			my_print('payload is nil')
			return nil, nil, nil
		end

		if (subDeviceId ~= nil) then
			local data = payload['data']
			if (data == nil) then
				my_print('subDeviceId is nil')
				return nil, nil, nil
			end
			local _subDeviceId = data['subDeviceId']
			if (_subDeviceId ~= subDeviceId) then
				return nil, nil, nil
			end
		end

		return js['fixedHeader'], js['variableHeader'], payload['context'], payload['data']
	end

	return nil, nil, nil
end


math.randomseed(os.time())

-- ------------------------------------------------------
function rspcmd(_id, _code) 
	local ret = {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"registerResp",
		time		=	time(), 
		deviceCode	=	get_devcode(),
		id		= 	_id,
		data		= 	{
			code		=	_code,
			deviceCode 	= get_devcode(),
			mac		= gwmac()
		}
	}
	return ret
end

function rspnull() 
	return {}
end

function ubus_send(to, msg) 
	local pkt={PKT = json.encode(msg)}
	--my_print('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' ' .. '\'' .. json.encode(pkt) .. '\'')
	--my_print('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' ' .. '\'' .. json.encode(pkt) .. '\'')
	--os.execute('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' \'' .. json.encode(pkt) .. '\'')
	my_print('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
	--os.execute('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
	luci.util.exec('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
end

function gwstatus() 
	ret = {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"reportAttribute",
		time		=	time(), 
		deviceCode	=	get_devcode(),
		data		= 	{
			mac		= gwmac(),
			attribute	= "gateway.status",
			value 		= {
	 			version 	= get_version(),
				model		= get_model(),
				factory		= get_factory(),
				current_time	= time(),
				uptime		= uptime(),
				wireless	= "",
				ethernet_ip	= "",
				uplinkType	= ""
			}
		}
	}
	return ret
end

function gwregister() 
	local devcode = fs.readfile('/proc/sys/kernel/random/uuid')
	devcode = string.gsub(devcode, "%s+", "")
	my_print('devcode is :' .. devcode)
	set_devcode(devcode)
	return {}

	--[[
	return {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"registerReq",
		time		=	time(), 
		data		= 	{
			mac		= gwmac(),
			license = get_factory(),
			factory = get_factory(),
		}
	}
	--]]
end

-- cache:
-- --------------------------------------------------------------------------------

-- cache:
-- --------------------------------------------------------------------------------
--[[
__cache = {
	devs = {
		'testdev' = {
			attrs = {
				attr1 = { value = '1', last = 0}
				attr2 = { value = 0,	 last = 1}
			}
		}
	}
	tasks = {
		'testtask' = {
			func		= 'funcname',
			arg			= {},
			start		= start,
			delt		= 5 * 60 * 1000
		}
	}
}
--]]
__cache = {}
function cache_load()
	local __s = fs.readfile("/tmp/._mqtt-gw_cache")
	__cache = json.decode(__s) or {}
end
function cache_save()
	local __s = json.encode(__cache) or '{}'
	fs.writefile("/tmp/._mqtt-gw_cache", __s)
end

-- cache dev
function cache_dev_value_get(dev, attr)
	if (__cache['devs'] == nil) then
		return nil, nil
	end
	if (__cache['devs'][dev] == nil) then
		return nil, nil
	end
	if (__cache['devs'][dev][attr] == nil) then
		return nil, nil
	end
	return __cache['devs'][dev][attr]['value'], __cache['devs'][dev][attr]['last']
end
function cache_dev_value_set(dev, attr, value)
	if (__cache['devs'] == nil) then
		__cache['devs'] = {}
	end
	if (__cache['devs'][dev] == nil) then
		__cache['devs'][dev] = {}
	end
	__cache['devs'][dev][attr] = { value = value, last = time() }
end

-- cache task
function cache_schedule_first_task_delay() 
	local min = nil
	for key, task in pairs(__cache['tasks']) do
		local now = time()
		local diff = task['start'] + task['delt'] - now
		if (min == nil or diff < min) then
			min = diff
		end
	end

	if (min == nil) then
		return nil
	end

	if (min < 0) then
		min = 0
	end

	return min
end
function cache_schedule_add(task,  delt, func, arg) 
	if (__cache['tasks'] == nil) then
		__cache['tasks'] = {}
	end
	__cache['tasks'][task] = {
		func		= func, 
		arg			= arg,
		start		= time(),
		delt		= delt,
		valid		= 1
	}
	local delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return
	end
	if (delay == 0) then
		delay = 1
	end
	schedule_after(delay)
end
function cache_schedule_del(task)
	if (__cache['tasks'] == nil) then
		return
	end
	__cache['tasks'][task] = nil

	local delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return
	end
	if (delay == 0) then
		delay = 1
	end
	schedule_after(delay)
end
function cache_schedule_func()
	if (__cache['tasks'] == nil) then
		return {}
	end

	local deltasks = {}
	for key, task in pairs(__cache['tasks']) do
		local now = time()
		local diff = task['start'] + task['delt'] - now
		if (diff < 0) then
			diff = 0
		end

		if (diff == 0) then
			local fname = task['func']
			local f = _G[fname]
			deltasks[key] = '1'
			__cache['tasks'][key]['valid'] = 0
			if (type(fname) == "string" and type(_G) == "table" and type(f) == "function")  then
				f(task['arg'])
			end
		end
	end

	for k,v in pairs(deltasks) do
			if (__cache['tasks'][k]['valid'] == 0) then
				my_print('-- del ' .. k)
				__cache['tasks'][k] = nil
			end
	end

	delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return {}
	end
	if (delay == 0) then
		delay = 1
	end

	schedule_after(delay)
	return {}
end


--: rpt handler ----------------------------------------------------------------------------
function register_handler(type, from, attr, mac, id, code, value) 
	return {}
end

function cmd_result_handler_add_sub_device(fixedHeader, variableHeader, context, data, type, from, attr, mac, id, code, value) 
	local traceId = context['traceId']
	if (traceId ~= id) then
		backup_cmd(fixedHeader, variableHeader, traceId, context['requestMethod'], context['requestTime'], context['deviceId'], context['deviceModel'], data)
		return {}
	end
		
	local code		= 0
	local message = 'OK'

	if (code == 104) then
		code = 104
		message = 'timeout'
	elseif (code == 99) then
		backup_cmd(fixedHeader, variableHeader, traceId, context['requestMethod'], context['requestTime'], context['deviceId'], context['deviceModel'], data)
		return {}
	else
		backup_cmd(fixedHeader, variableHeader, traceId, context['requestMethod'], context['requestTime'], context['deviceId'], context['deviceModel'], data)
		return {}
	end

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	--subDeviceId		= _mac,

	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)
	return {}
end
function cmd_result_hanlder(type, from, attr, mac, id, code, value) 
	--[[
	local fixedHeader, variableHeader, context, data = fetch_one_cmd('deviceUpgrade', get_gateway_device_id(), nil)
	if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
		-- my_print('no command for addSubDevice, deviceId is ' .. get_gateway_device_id() .. ', traceId is ' .. id)
		fixedHeader, variableHeader, context, data = fetch_one_cmd('addSubDevice', get_gateway_device_id(), nil)
		if (fixedHeader ~= nil and variableHeader ~= nil and context ~= nil and data ~= nil) then
			return cmd_result_handler_add_sub_device(fixedHeader, variableHeader, context, data, type, from, attr, mac, id, code, value)
		else
			return {}
		end
	end
	

	local traceId = context['traceId']
	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			context = {
				traceId				= traceId,
				requestTime		= time_ms(),
				requestMethod	= 'upgradeNotify',
				deviceId			= get_gateway_device_id(),
				deviceModel		= get_model(),
			},
			data = {
				upgradeState	= tostring(code)
			}
		}
	}

	-- 207 download ok
	-- 208 checksum ok
	-- 255 failed
	-- 1-100 download progress
	if (code == 97) then -- start upgrade download o, md5 ok
		return {}
	elseif (code == 98) then -- upgrade success
		ret['payload']['data']['upgrade'] = 207
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		
		ret['payload']['data']['upgrade'] = 208
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
			
	elseif (code == 108) then -- md5sum failed
		ret['payload']['data']['upgrade'] = 207
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		
		ret['payload']['data']['upgrade'] = 255
		sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
	elseif (code == 109) then -- download failed
		ret['payload']['data']['upgrade'] = 1
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		
		ret['payload']['data']['upgrade'] = 255
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
	elseif (code == 110) then -- upgrade failed
		ret['payload']['data']['upgrade'] = 255
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
	elseif (code == 130) then -- deleted
	elseif (code == 131) then -- version error
	elseif (code == 132) then -- low power
	elseif (code == 133) then -- off line
	end
	--]]
	return {}
end

function pir_nomotion_delay_5min_rpt(arg) 
	local ret = arg
	local mac = ret['data']['subDeviceId']
	local _value, _last = cache_dev_value_get(mac, 'NotifyValue')
	ret['data']['state'] = 0
	_value['state'] = 0

	my_print('NotifyValue Update -> ' .. json.encode(_value))
	cache_dev_value_set(mac, 'NotifyValue', _value)
	
	local sret = json.encode(arg)
	my_print('sret is' .. sret)
	mqtt_publish('report', sret, #sret)
	return {}
end

function report_handler(type, from, attr, mac, id, code, value) 
	my_print('value is ' .. json.encode(value))

	if ( attr == 'gateway.status') then
		luci.util.exec('touch /tmp/._rpt_gateway_status')
		return {}
	elseif (attr == 'gateway.state.change') then
		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'deviceStateChangeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model()
			},
			data = {}
		}
		local cvalue = {
			networkType			= get_gateway_network_type(),
			networkId				= get_gateway_network_id(),
			networkRssi			= get_gateway_network_strength(),
			powerType				= get_gateway_power_type(),
			armMode					= get_gateway_arm_mode()
		}
		local _value, _last = cache_dev_value_get(gwmac(), 'NotifyValue')
		local _is_change = false
		
		if (_value == nil) then
			_is_change = true
			ret['data'] = cvalue
		else 
			if (_value['networkType'] ~= cvalue['networkType'] or 
					_value['networkId']		~= cvalue['networkId']   or 
					_value['networkRssi'] ~= cvalue['networkRssi']) then
				_is_change = true
				ret['data']['networkType']	= cvalue['networkType']
				ret['data']['networkId']		= cvalue['networkId']
				ret['data']['networkRssi']	= cvalue['networkRssi']
			end

			if (_value['powerType'] ~= cvalue['powerType']) then
				_is_change = true
				ret['data']['powerType'] = cvalue['powerType']
			end

			if (_value['armMode'] ~= cvalue['armMode']) then
				_is_change = true
				ret['data']['armMode'] = cvalue['armMode']
			end
		end

		if (_is_change == true) then
			cache_dev_value_set(gwmac(), 'NotifyValue', cvalue)
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end

		return {}
	elseif (attr == 'gateway.set.config.mode') then
		local data = {
			configType = 1
		}
		set_device_config_mode(data) 
		return {}
	elseif (attr == 'gateway.btn.factory_reset') then
		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'manualResetDeviceNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model()
			},
			data = {
				yum = "yum"
			}
		}
	
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		return {}
	elseif (attr == 'gateway.mqtt.reconnect') then
		--my_print('mqtt reconnect!!!')
		--mqtt_reconnect(1)
		return {}
	elseif (attr == 'gateway.test.xxx') then
		local msg = {
			context	= {
				traceId				= get_uuid(),
				requestTime		= 1576505177,
				requestMethod	= 'deviceUpgrade',
				deviceId			= 'dsb66d9c8e656edcaa1eb6776a5b923a5a',
				deviceModel		= 'hub_ds'
			},
			data = {
				upgradeUrl		= 'http://203.189.235.135:4999/Public/Uploads/2020-02-14/5e4661ff04d4b.gz',
				firmVersion		= '1.0.20',
				md5sum				= '52dc4b69c3143683f130d0c6a817825c'
			}
		}
		cmdin(json.encode(msg))
		return {}
	elseif (attr == 'gateway.heartbeat') then
		-- subDeviceList is a [no] segment
		local ret = {
				context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'deviceRegularNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model()
				},
				data = {
					battery					= 100,
					networkRssi			= get_gateway_network_strength(),
					networkType			= get_gateway_network_type(),
					networkId				= get_gateway_network_id()
				}
			}
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		return {}
	elseif (attr == 'gateway.wifi.config.process') then
		local configId	= value['configId']
		local step			= value['step']
		local code			= value['code']
		local msg				= value['msg']
		local debugInfo = value['debugInfo']
		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'configStateNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model()
			},
			data = {
				configId				= configId,
				step						= step,
				code						= code, 
				msg							= msg,
				debugInfo				= debugInfo
			}
		}
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)

		if (step == 'DNS_RESOLVE') then
			ret = {
				context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'configStateNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model()
				},
				data = {
					configId				= configId,
					step						= 'CONNECT_SERVER',
					code						= code, 
					msg							= msg,
					debugInfo				= debugInfo
				}
			}
			sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end

		local notify2app = true
		if (notify2app == true) then
			local Protocol			= '01'
			local ServiceType		= '01'
			local SequenceId		= '0001'
			-- bit[7~4]: security, 0000 none, 0001 RSA1024/AES128/CBC-PKCS1Padding
			-- bit[3~0]: Version, 0010
			local Flag					= '12'
			local OpCode				= '8014'
			local StatusCode		= '00'
		
			ret = {
				Protocol		= Protocol,
				ServiceType	= ServiceType,
				SequenceId	= SequenceId,
				Length			= 0,
				Flag				= Flag,
				OpCode			= OpCode,
				StatusCode	= code,
				Payload			= {
					configId	= configId,
					step			= step,
					code			= code,
					msg				= msg,
					debugInfo = debugInfo
				}
			}
			ret.Length			= #json.encode(ret.Payload)
			sret = json.encode(ret)
			my_print('sret is' .. sret)
			tcp_cfg_out('c', sret, #sret)
		end

		
		if (step == 'DNS_RESOLVE') then
			cache_schedule_del('setWifiExpiredTask')
			luci.util.exec('mkdir -p /etc/config/dusun/mqtt-gw; echo 0 > /etc/config/dusun/mqtt-gw/configMode')

			local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
			if (mtksdk == '1') then
				luci.util.exec('[ -e /tmp/._mqttconnect ] && gpio l 61 0 4000 0 1 4000 || gpio l 61 4000 0 1 0 4000')
				luci.util.exec('gpio l 62 4000 0 1 0 4000')
			else
				luci.util.exec('echo none > /sys/class/leds/errled/trigger; echo 0 > /sys/class/leds/errled/brightness')
			end
		end

		return {}
	elseif (attr == 'mod.device_list') then
		-- tid='ZB3_' .. devStr .. '_' .. mac
		fs.writefile("/tmp/.zb_devicelist",json.encode(value))
		local subDeviceList = {}
		local device_list=value['device_list']
		for i=1,#device_list do
			local ji=device_list[i]

			--
			-- my_print(ji)
			-- {"mac":"000d6f000be63f63","type":"1218","subtype":"","ModelStr":"SmartPlug","battery":100,"online":1,"version":"1.2",
			--	"model":"Dusun","epList":[{"ep":"01","type":"1218"}]}
			
			local _mac				= ji.mac
			local type			= ji.type
			local subtype		= ji.subtype
			local modelstr	= ji.ModelStr
			modelstr	= devstr_search_by_modelstr(modelstr)
			local battery		= ji.battery
			local online		= ji.online
			local version		= ji.version
			local model			= ji.model
			local state			= ji.state or '0'
			local mcuver		= ji.mcuversion == '' and nil or ji.mcuversion
			local zigver		= ji.zigversion == '' and nil or ji.zigversion
			local tamper		= 0
			local autoLockTimeInSec = ji.autoLockTimeInSec

			state = tonumber(state)
			if (modelstr == 'ask110') then
				state = nil
			end
		
			if (modelstr ~= 'UnknownDevice') then
					subDeviceList[#subDeviceList+1] = {
					subDeviceId =	_mac,
					deviceModel	= modelstr,
					firmVersion = version,
					firmInfo		= mcuver == nil and nil or {
						{
							firmName		= 'mcu',
							firmVersion	= mcuver
						},
						{
							firmName		= 'zigbee',
							firmVersion	= zigver
						}
					},
					sn					= _mac,
					online			= online,
					state				= state,
					battery			= battery,
					autoLockTimeInSec = autoLockTimeInSec
				}
			end
			--state				= tonumber(state),
			--voltage			= battery
			--tamper			= tamper,
		end

		my_print(json.encode(subDeviceList))
		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getDeviceInfo', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			my_print('no command for getDeviceInfo, deviceId is ' .. get_gateway_device_id() .. ', traceId is ' .. id)

			connect_cloud_flag = string.gsub(luci.util.exec('[ -e /tmp/._rpt_gateway_status ] && echo 1 || echo 0'), "%s+", "")
			my_print('connect_cloud_flag ' .. connect_cloud_flag)
			if (connect_cloud_flag == '0') then
				my_print('no connect cloud flag ')
				
				--[[
				for i=1,#(subDeviceList) do
					subDeviceList[i]['deviceModel'] = nil
					subDeviceList[i]['firmVersion'] = nil
					subDeviceList[i]['sn'] = nil
					subDeviceList[i]['online'] = nil
					subDeviceList[i]['state'] = nil
				end
				--]]
				local ret = {
					context = {
						traceId					= get_uuid(),
						requestTime			= time_ms(),
						requestMethod		= 'deviceRegularNotify',
						deviceId				= get_gateway_device_id(),
						deviceModel			= get_gateway_model()
					},
					data = {
						battery					= 100,
						networkRssi			= get_gateway_network_strength(),
						networkType			= get_gateway_network_type(),
						networkId				= get_gateway_network_id(),
					}
					--subDeviceNumber	= #device_list,
					--subDeviceList		= subDeviceList,
				}
				--local sret = json.encode(ret)
				--sret = string.gsub(sret, '{}', '[]')
				--my_print('sret is' .. sret)
				--mqtt_publish('report', sret, #sret)
				return {}
			end


			if (fs.access('/etc/config/dusun/mqtt-gw/up_record')) then
				local __up_record_s = fs.readfile("/etc/config/dusun/mqtt-gw/up_record")
				local jret = json.decode(__up_record_s) or nil
				if (jret ~= nil) then
					my_print('sret is' .. __up_record_s)
					mqtt_publish('report', __up_record_s, #__up_record_s)
				end
			end
	
			local ret = {
				context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'deviceConnectNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model()
				},
				data = {
					firmVersion			= get_gateway_version(),
					subDeviceNumber	= #device_list,
					battery					= get_gateway_battery(),
					powerType				= get_gateway_power_type(),
					armMode					= get_gateway_arm_mode(),
					sn							= get_gateway_sn(),
					networkType			= get_gateway_network_type(),
					networkRssi		  = get_gateway_network_strength(),
					networkId				= get_gateway_network_id(),
					subDeviceList		= subDeviceList
				}
			}
			--deviceId				= get_gateway_device_id(),			
			--deviceModel			= get_gateway_model(),
			--networkStrength = get_gateway_network_strength(),
			local sret = json.encode(ret)
			sret = string.gsub(sret, '{}', '[]')
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
			luci.util.exec('rm -rf /tmp/._rpt_gateway_status');
			return {}
		end

		local message = 'OK'
		local code = 0
		local traceId = context['traceId']

		local dgstr			= dginfo('a')
		local dgjson		= json.decode(dgstr)

		local nrssi			= dgjson['csq']
		my_print('nrssi:' .. nrssi)
		nrssi = luci.util.exec('echo -n' .. nrssi .. ' | cut -d "," -f 1 | cut -d ":" -f 2')
		local ntype = get_gateway_network_type()
		if (ntype == '4G') then
			if (nrssi == '') then
				nrssi = 0
			else
				nrssi = tonumber(nrssi)
			end
		else
			nrssi = luci.util.exec('iwpriv ra0 stat | grep RSSI | xargs | cut -d " " -f 3')
			nrssi = tonumber(nrssi)
		end
		
		local nquality	= dgjson['qcsq']
		my_print('nquality:' .. nquality)
		nquality = luci.util.exec('echo ' .. nquality .. ' | cut -d "," -f 4')
		nquality = string.gsub(nquality, "%s+", "")
		if (#nquality == 0) then
			nquality = nil
		else
			nquality = tonumber(nquality)
		end

		local nact			= dgjson['qnwinfo']
		my_print('nact:' .. nact)
		nact = luci.util.exec('echo ' .. nact .. ' | cut -d "," -f 1 | cut -d ":" -f 2')
		nact = string.gsub(nact,"%s+", "")
		if (#nact == 0) then
			nact = nil
		end

		local nband			= dgjson['qnwinfo']
		my_print('nband:' .. nband)
		nband = luci.util.exec('echo ' .. nband .. ' | cut -d "," -f 3')	
		nband = string.gsub(nband,"%s+", "")
		if (#nband == 0) then
			nband = nil
		end

		local nid				= dgjson['qspn']
		my_print('nid :' .. nid)
		nid = luci.util.exec('echo ' .. nid .. '  | cut -d "," -f 1 | cut -d ":" -f 2')
		nid = string.gsub(nid, "%s+", "")
		if (#nid == 0) then
			nid = nil
		end

		--my_print('nrssi : ' .. nrssi .. ', nquality:' .. nquality .. ',nact:' .. nact .. 'nband:' .. nband)

		--[[
		local ret = {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					deviceId				= get_gateway_device_id(),			
					deviceModel			= get_gateway_model(),
					firmVersion			= get_gateway_version(),
					subDeviceNumber	= #device_list,
					battery					= get_gateway_battery(),
					powerType				= get_gateway_power_type(),
					armMode					= get_gateway_arm_mode(),
					sn							= get_gateway_sn(),
					networkType			= get_gateway_network_type(),
					networkId				= get_gateway_network_id(),
					networkRssi		  = get_gateway_network_strength(),
					networkQuality	= get_gateway_snr(),
					networkAct			= get_gateway_qnwact(),
					networkBand			= get_gateway_qnwband(),
					subDeviceList		= subDeviceList
				}
			}
		}
		--]]
		local ret = {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					deviceId				= get_gateway_device_id(),			
					deviceModel			= get_gateway_model(),
					firmVersion			= get_gateway_version(),
					subDeviceNumber	= #device_list,
					battery					= get_gateway_battery(),
					powerType				= get_gateway_power_type(),
					armMode					= get_gateway_arm_mode(),
					sn							= get_gateway_sn(),
					networkType			= get_gateway_network_type(),
					networkId				= get_gateway_network_id(),
					networkRssi		  = nrssi,
					subDeviceList		= subDeviceList
				}
			}
		}
		--networkQuality	= nquality,
		--networkAct			= nact,
		--networkBand			= nband,

		if (nact ~= nil) then
			ret['payload']['result']['networkAct'] = nact
		end
		if (nband ~= nil) then
			ret['payload']['result']['networkBand'] = nband
		end
		if (nquality ~= nil) then
			ret['payload']['result']['networkQuality'] = nquality
		end
		if (nid ~= nil) then
			ret['payload']['result']['networkId'] = nid
		end

		
		
		--networkStrength = get_gateway_network_strength(),
		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is' .. sret)
		mqtt_publish('response', sret, #sret)
		return {}	
	elseif (attr == 'mod.new_device_added') then
		local _mac			= value.mac
		local type			= value.type
		local subtype		= value.subtype
		local modelstr	= value.ModelStr
		modelstr	= devstr_search_by_modelstr(modelstr)
		local battery		= value.battery
		local online		= value.online
		local version		= value.version
		local model			= value.model
		local state			= value.state or '0'
		local mcuver		= value.mcuversion == '' and nil or value.mcuversion
		local zigver		= value.zigversion == '' and nil or value.zigversion
		local tamper		= modelstr == 'asl120' and nil or 0
		local autoLockTimeInSec = value.autoLockTimeInSec
		local _devid		= _mac
		if (modelstr == 'smart_lock_ds' or modelstr == 'asl120') then
			_devid = 'addlock'
		else
			_devid = _mac
		end
		local fixedHeader, variableHeader, context, data = fetch_one_cmd('addSubDevice', get_gateway_device_id(), _devid)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			fixedHeader, variableHeader, context, data = fetch_one_cmd('addSubDevice', get_gateway_device_id(), 'addlock')
			if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
				my_print('no command for addSubDevice, deviceId is ' .. get_gateway_device_id() .. ', traceId is ' .. id)
				if (modelstr == 'UnknownDevice') then
					os.execute('dusun_ucmd.sh remove ' .. _devid)
				end
				return {}
			end
		end
		local message = 'OK'
		local code = 0
		local traceId = context['traceId']

		cache_schedule_del('add_subdevice_' .. _mac)
		cache_schedule_del('add_subdevice_' .. 'addlock')
		local ret = {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					subDeviceId		= _mac,
					deviceModel		= modelstr,
					firmVersion		= version,
					firmInfo		= mcuver == nil and nil or {
						{
							firmName		= 'mcu',
							firmVersion	= mcuver
						},
						{
							firmName		= 'zigbee',
							firmVersion	= zigver
						}
					},
					sn						= _mac,
					online				= online,
					state					= tonumber(state),
					tamper				= tamper,
					battery				= battery,
					autoLockTimeInSec = autoLockTimeInSec
				}
			}
		}
		--voltage				= battery
		--tamper			= tamper,
		if (modelstr == 'ask110') then
				ret['payload']['result']['state'] = nil
		end
		
		local sret = json.encode(ret['payload'])
		my_print('sret is' .. sret)
		mqtt_publish('response', sret, #sret)
		return {}
	elseif (attr == 'mod.device_deleted') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'manualResetDeviceNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model()
			},
			data = {
				subDeviceId = mac
			}
		}
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		return {}
	elseif (attr == 'device.lock.one_touch_lock') then
		local modelstr	= value.ModelStr
		modelstr				= devstr_search_by_modelstr(modelstr)
		local enable		= value.enable or '0'
		local uuid			= value.uuid
		if (uuid == nil) then
			return {}
		end

		local traceId = string.sub(uuid,5, -1)
		local message	= 'OK'
		local ret=  {
			payload					= {
				traceId		= traceId,
				code			= 0,	
				msg				= message,
				result		= {
					enable = enable
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return {}
	elseif (attr == 'device.signal') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local battery		= value['battery'] or 100
		local version		= value['version']
		local online		= value['online']
		local rssi			= value['rssi']
		local uuid			= value['uuid']

		local traceId = string.sub(uuid, 5, -1)
		local message	= 'OK'
		local ret=  {
			payload					= {
				traceId		= traceId,
				code			= 0,	
				msg				= message,
				result		= {
					subDeviceId = mac,
					deviceModel	= modelstr,
					online			= online,
					signalQuality = rssi
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
	
		return {}
	elseif (attr == 'device.lock.report_system_status') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local systemStatus = value['systemStatus']
		local pass = 0

		if (systemStatus == 'systemLock') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'lockKeypadUseNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= modelstr,
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					eventType				= 2,
					eventResult		  = pass,
					eventTime				= time_ms(),
					eventMsg				= {
					}
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		elseif (systemStatus == 'tramp') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'subDeviceStateChangeNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model(),
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					tamper					= 1
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		elseif (systemStatus == 'systemDisLock') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'lockKeypadUseNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= modelstr,
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					eventType				= 3,
					eventResult		  = pass,
					eventTime				= time_ms(),
					eventMsg				= {
					}
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		elseif (systemStatus == 'distramp') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'subDeviceStateChangeNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model(),
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					tamper					= 0
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		elseif (systemStatus == 'release') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'subDeviceStateChangeNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model(),
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					tamper					= 0
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end
		return {}
	elseif (attr == 'device.status') then
		--{"type":"1210","subtype":"","ModelStr":"Leak","rssi":0,"battery":57,"online":1,"time":"1588224066","zone":"Leak","version":"1.0","ep":242}
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local battery		= value['battery'] or 100
		local version		= value['version']
		local online		= value['online']
		local voltage		= 0
		if (battery <= 0) then
			return {}
		end
		if (battery < 30) then
			voltage = 1
		else
			voltage = 0
		end
	
		local ret = {
				context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'subDeviceStateChangeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model(),
			},
			data = {
				subDeviceId			= mac,
				deviceModel			= modelstr,
			}
		}
		local cvalue = {
				subDeviceId			= mac,
				deviceModel			= modelstr,
				online					= online,
				battery					= battery,
				voltage					= voltage
		}
		local _value, _last = cache_dev_value_get(mac, 'NotifyValue')
		local _is_change = false

		if (_value == nil) then
			_is_change = true
			ret['data'] = cvalue
			_value = cvalue
		else
			if (_value['battery'] ~= cvalue['battery']) then
				_is_change = true
				ret['data']['battery'] = cvalue['battery']
				_value['battery'] = cvalue['battery']
			end

			if (_value['online'] ~= cvalue['online']) then
				_is_change = true
				ret['data']['online'] = cvalue['online']
				_value['online'] = cvalue['online']
			end

			if (_value['voltage'] ~= cvalue['voltage']) then
				_is_change = true
				ret['data']['voltage'] = cvalue['voltage']
				_value['voltage'] = cvalue['voltage']
			end

		end

		if (_is_change == true) then
			my_print('NotifyValue Update -> ' .. json.encode(_value))
			cache_dev_value_set(mac, 'NotifyValue', _value)
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end
		return {}
	elseif (attr == 'device.arm_status') then
		-- {"id": 3, "ep": 1, "zone": "arm", "code": "123456", "battery": 100, "value": 0, "ModelStr": "ZB-KeypadGeneric-D0000"}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local zone			= value['zone']
		local amode		  = value['value']
		local battery		= value['battery'] or 100
		local code			= value['code'] or nil
		local codeId		= value['codeId'] or nil
		if (amode == 3) then
			amode = 2
		end

		if (zone ~= 'arm') then
			return {}
		end
		--[[
		if (get_gateway_arm_mode() == amode) then
			return {}
		end
		--]]

		local ret = {
				context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'keypadChangeModeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model(),
			},
			data = {
				deviceId				= mac,
				deviceModel			= modelstr,
				mode						= amode,
				codeId					= codeId
			}
		}

		my_print('keypad change notify -> ' .. json.encode(ret.data))
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		return {}
	elseif (attr == 'device.zone_status') then
		-- {"ep":242,"value":"1","ModelStr":"Leak","battery":84,"zone":"leak"}
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local zone			= value['zone']
		local valstr		= value['value']
		local battery		= value['battery'] or 100
		local voltage		= 0
		if (modelstr == 'ask110') then
			return {}
		end

		if (valstr == '4') then
			valstr = '0'
		elseif (valstr == '5') then
			valstr = '1'
		end

		--[[
		if (zone == 'pir' or zone == 'door') then
			local amode			= get_gateway_arm_mode()
			if (amode == 0) then
				return {}
			end
		end
		--]]

		if (battery <= 0) then
			return {}
		end
		if (battery < 30) then
			voltage = 1
		else
			voltage = 0
		end
		
		local zonestr		= {
			door			= 'DoorStatus',
			co				= 'GasStatus',
			panicbtn	= 'EmergCaStatus',
			smoke			= 'SmokeStatus',
			pir				= 'PIRStatus',
			leak			= 'LeakStatus',
		}
		local fnstr			= zonestr[zone] or zone
	
		local ret = {
				context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'subDeviceStateChangeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model(),
			},
			data = {
				subDeviceId			= mac,
				deviceModel			= modelstr
			}
		}
		local cvalue = {
				subDeviceId			= mac,
				deviceModel			= modelstr,
				online					= 1,
				state						= tonumber(valstr),
				tamper					= modelstr == 'asl120' and nil or 0,
				deviceModel			= modelstr,
				battery					= battery,
				voltage					= voltage
		}

		local __LastState, __last = cache_dev_value_get(mac, 'LastState')
		cache_dev_value_set(mac, 'LastState', cvalue['state'])

		local _value, _last = cache_dev_value_get(mac, 'NotifyValue')
		local _is_change = false

		local amode			= get_gateway_arm_mode()
		if (amode == 0 and zone == 'pir') then
			if (_value ~= nil and _value['state'] == 1 and cvalue['state'] == 0) then
				if (__LastState ~= nil and __LastState == 1) then
					my_print('delay 5min to rpt pir no motion')
					-- start 5min schedule rpt
					cache_schedule_add('pir_nomotion_delay_5min_rpt_' .. mac, 5*60, 'pir_nomotion_delay_5min_rpt', ret)
				else
					my_print('no action')
				end
				return {}
			else
				-- delete 5min schedule rpt
				my_print('delete 5min delay pir no motion task')
				cache_schedule_del('pir_nomotion_delay_5min_rpt_' .. mac)
			end
		else
				cache_schedule_del('pir_nomotion_delay_5min_rpt_' .. mac)
		end
		
		if (_value == nil) then
			_is_change = true
			ret['data'] = cvalue
			_value = cvalue
		else
			if (_value['state'] ~= cvalue['state']) then
				_is_change = true
				ret['data']['state'] = cvalue['state']
				_value['state'] = cvalue['state']
			end

			if (_value['battery'] ~= cvalue['battery']) then
				_is_change = true
				ret['data']['battery'] = cvalue['battery']
				_value['battery'] = cvalue['battery']
			end

			if (_value['voltage'] ~= cvalue['voltage']) then
				_is_change = true
				ret['data']['voltage'] = cvalue['voltage']
				_value['voltage'] = cvalue['voltage']
			end
		end

		--firmVersion			= '1.0'
		if (_is_change == true) then
			my_print('NotifyValue Update -> ' .. json.encode(_value))
			cache_dev_value_set(mac, 'NotifyValue', _value)

			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end
		return {}
	elseif (attr == 'device.temperature') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local unit			= value['unit']
		local valstr		= value['value']

		return {}
	elseif (attr == 'device.humidity') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local unit			= value['unit']
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.energy') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.power') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.voltage') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.current') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.switch.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.meter.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		return {}
	elseif (attr == 'device.light.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.lock.check_record') then
		--{"passType": 0, "src": "manual", "passId": 65535, "pass": 1, "ModelStr": "YRD216 PBDB", "time": 4204768, "evt": "open", "battery": 100, "zone": "YRD216 PBDB"}, "mac": "000d6f001118d477"}
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local passId		= value['passId']
		local passType	= value['passType']
		local passVal1	= value['passVal1'] or '******'
		local timeval		= string.format('%d', value['time'])
		local pass			= string.format('%d', value['pass'])
		local src				= value['src']
		local	evt				= value['evt']
		local battery		= value['battery']
		local voltage		= 0
		if (battery < 30) then
			voltage = 1
		else
			voltage = 0
		end

		local state			= 0
		if (evt == 'open') then
			state = 1
		else
			state = 0
		end

		local useType		= nil
		if (src == 'keypad') then
			useType = 2
		elseif (src == 'rf') then
			useType = 1
		elseif (src == 'onetouch') then
			useType = 5
		elseif (src == 'thumbturn/key') then
			useType = 3
		elseif (src == 'rfid') then
			useType = 5
		elseif (src == 'auto') then
			useType = 4
		end

		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'subDeviceStateChangeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model(),
			},
			data = {
				subDeviceId			= mac,
				deviceModel			= modelstr
			}
		}
		local cvalue = {
				subDeviceId			= mac,
				deviceModel			= modelstr,
				online					= 1,
				state						= state,
				tamper					= modelstr == 'asl120' and nil or 0,
				deviceModel			= modelstr,
				battery					= battery,
				voltage					= voltage,
				entryCodeId			= passId,
				useType					= useType
		}
		local _value, _last = cache_dev_value_get(mac, 'NotifyValue')
		local _is_change = false
		
		my_print('--- 1')
		if (_value == nil) then
		my_print('--- 2')
			_is_change = true
			ret['data'] = cvalue
			_value = cvalue
		else 
			my_print('----' .. json.encode(cvalue))
			my_print('----' .. json.encode(_value))
			--[[
			if (_value['state'] ~= cvalue['state']) then
				_is_change = true
				ret['data']['state'] = cvalue['state']
				_value['state'] = cvalue['state']
			end
			--]]
			if (_value['battery'] ~= cvalue['battery']) then
				_is_change = true
				ret['data']['battery'] = cvalue['battery']
				_value['battery'] = cvalue['battery']
			end
			if (_value['voltage'] ~= cvalue['voltage']) then
				_is_change = true
				ret['data']['voltage'] = cvalue['voltage']
				_value['voltage'] = cvalue['voltage']
			end

			_is_change = true
			ret['data']['entryCodeId'] = cvalue['entryCodeId']
			_value['entryCodeId'] = cvalue['entryCodeId']

			_is_change = true
			ret['data']['useType'] = cvalue['useType']
			_value['useType'] = cvalue['useType']
		end
	
		if (_is_change == true) then
			my_print('NotifyValue Update -> ' .. json.encode(_value))
			cache_dev_value_set(mac, 'NotifyValue', _value)
			local sret = json.encode(ret)
			my_print('sret is ' .. sret)
			mqtt_publish('report', sret, #sret)
		end
	
		if (src == 'keypad') then
			local ret = {
					context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'lockKeypadUseNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= modelstr,
				},
				data = {
					subDeviceId			= mac,
					deviceModel			= modelstr,
					eventType				= 1,
					eventResult		  = pass,
					eventTime				= time_ms(),
					eventMsg				= {
						entryCodeId		= passId
					}
				}
			}
			--firmVersion			= '1.0'
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)
		end

		if (src == 'rf') then
			local fixedHeader, variableHeader, context, data = fetch_one_cmd('setLockState', get_gateway_device_id(), mac)
			if (fixedHeader == nil or variableHeader == nil or context == nil or data == nil) then
				return {}
			end

			local traceId = context['traceId']
			local message	= 'OK'
			local ret=  {
				fixedHeader			= fixedHeader,
				variableHeader	= variableHeader,
				payload					= {
					traceId		= traceId,
					code			= 0,	
					msg				= message,
					result		= {
					}
				}
			}
			local sret = json.encode(ret['payload'])
			my_print('sret is ' .. sret)
			mqtt_publish('response', sret, #sret)
		end

		return {}

	elseif (attr == 'device.lock.password_adddel_result') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local operation	= value['operation']
		local passId		= string.format('%d', value['passId'])
		local code			= string.format('%d', value['code'])

		if (passId == '65488') then
			if (code == '0') then
				return {}
			else
				my_print('ignore the 65488 one key open')
				return {}
			end
		else
			return {}
		end
	elseif (attr == 'device.lock.lockstate') then
	--TODO:
	--{"attribute": "device.lock.lockstate", "mac": "000d6f00108de6a0", "ep": 1, "value": {"ep": 1, "value": 1, "battery": 95, "ModelStr": "YRD216 PBDB", "zone": "YRD216 PBDB"}
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local zone			= 'leak'
		local valstr		= value['value']
		local battery		= value['battery'] or 100
		local voltage		= 0
		if (battery <= 0) then
			return {}
		end
		if (battery < 30) then
			voltage = 1
		else
			voltage = 0
		end
		
		--[[
		if (valstr ~= 1) then
			return {}
		end
		--]]
		
		local state = 0
		if (valstr == 1) then
			state = 0
		else
			state = 1
		end

		--[[
		if (modelstr == 'asl120') then
			return {}
		end
		--]]
	
		
		local zonestr		= {
			door			= 'DoorStatus',
			co				= 'GasStatus',
			panicbtn	= 'EmergCaStatus',
			smoke			= 'SmokeStatus',
			pir				= 'PIRStatus',
			leak			= 'LeakStatus',
		}
		local fnstr			= zonestr[zone] or zone
	
		local ret = {
				context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'subDeviceStateChangeNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model(),
			},
			data = {
				subDeviceId			= mac,
				deviceModel			= modelstr
			}
		}
		--firmVersion			= '1.0'

		local cvalue = {
			subDeviceId			= mac,
			deviceModel			= modelstr,
			online					= 1,
			state						= state,
			tamper					= modelstr == 'asl120' and nil or 0,
			deviceModel			= modelstr,
			battery					= battery,
			voltage					= voltage,
			entryCodeId			= 65535,
			useType					= 4
		}		
		local _value, _last = cache_dev_value_get(mac, 'NotifyValue')
		local _is_change = false
		if (_value == nil) then
			_is_change = true
			ret['data'] = cvalue
			_value = cvalue
		else 
			if (_value['battery'] ~= cvalue['battery']) then
				_is_change = true
				ret['data']['battery']	= cvalue['battery']
				_value['battery'] = cvalue['battery']
			end

			if (_value['voltage'] ~= cvalue['voltage']) then
				_is_change = true
				ret['data']['voltage']	= cvalue['voltage']
				_value['voltage'] = cvalue['voltage']
			end

			--if (_value['state'] ~= cvalue['state'] or modelstr == 'asl120' ) then
			if (_value['state'] ~= cvalue['state']) then
				_is_change = true
				cvalue['useType'] = _value['useType'];
				cvalue['entryCodeId'] = _value['entryCodeId'];

				ret['data']['state'] = cvalue['state']
				ret['data']['useType'] = cvalue['useType']
				ret['data']['entryCodeId'] = cvalue['entryCodeId']

				_value['state'] = cvalue['state']
			end
		end
		
		if (_is_change == true) then
			my_print('NotifyValue Update -> ' .. json.encode(_value))
			cache_dev_value_set(mac, 'NotifyValue', _value)
			if (state == 0 or modelstr == 'asl120') then
				local sret = json.encode(ret)
				my_print('sret is' .. sret)
				mqtt_publish('report', sret, #sret)
			end
		end
		return {}
	elseif (attr == 'device.keypad.set_pass') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('setKeypadCode', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local result	= value['result']
		local codeId	= value['codeId']
		local code		= value['code']

		local traceId = context['traceId']
		local message = 'OK'
		if (result == 1) then
			message = 'password repeat!'
			result  = -53002101
		elseif (result == 2) then
			message = 'password full!'
			result  = -53002102
		end
		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					codeId	= codeId
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.keypad.del_pass') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('deleteKeypadCode', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local result	= value['result']
		local codeId	= value['codeId']

		local traceId = context['traceId']
		local message = 'OK'

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					codeId	= codeId
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.keypad.qry_pass') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getKeypadCode', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local result	= value['result']
		local codeId	= value['codeId']
		local code		= value['code']

		local traceId = context['traceId']
		local message = 'OK'

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					codeId	=  codeId,
					code		=	 code
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.keypad.qry_stat') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getKeypadInfo', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'

		local fwver		= value['version']
		local online	= value['online']
		local tamper	= value['tamper']
		local battery	= value['battery']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					firmVersion	= fwver,
					online			= online,	
					tamper			= tamper,
					battery			= battery
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.siren.relative') then
		-- get
		-- {"relative_list" : {"ModelStr":"", "mac":"", "active":""}}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getSirenEffectedDevices', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local rlist		= value['relative_list']

		local effectedDeviceIdList = {}
		for k, v in pairs(rlist) do
			effectedDeviceIdList[#effectedDeviceIdList+1] = {}
			effectedDeviceIdList[#effectedDeviceIdList]['deviceId'] = v['mac']
			effectedDeviceIdList[#effectedDeviceIdList]['deviceModel']	= v['ModelStr']
			effectedDeviceIdList[#effectedDeviceIdList]['active']	= v['active']
		end

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					effectedDeviceIdList = effectedDeviceIdList
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.siren.voice_level') then
		-- get
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getSirenVolume', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local volume	= value['value']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					volume	= volume
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()

	elseif (attr == 'device.siren.auto_off') then
		-- get
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getSirenAutoOff', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local auto_off= value['value']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					autoOff = auto_off
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()

	elseif (attr == 'device.repeater.schedules') then
		-- get
		-- {"schedule_list" : {"id":"", "repeat":"", "startTime":"", ""timeZone":"", "enabled":""}}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getRepeaterSchedule', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local	slist		= value['schedule_list']

		local schedules = {}
		for k, v in pairs(slist) do
			schedules[#schedules+1] = {}
			schedules[#schedules]['id']					= v['id']
			schedules[#schedules]['repeat']			= v['repeat']
			schedules[#schedules]['startTime']	= v['startTime']
			schedules[#schedules]['timeZone']		= v['timeZone']
			schedules[#schedules]['enabled']		= v['enabled']
			schedules[#schedules]['act']				= v['act']
		end

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					schedules = schedules
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.add_schedule') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('addRepeaterSchedule', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local result	= value['result']
		local	id			= value['id']

		local traceId = context['traceId']
		local message = 'OK'
		if (result ~= 0) then
			message = 'schdule full!'
			result  = -50352000
		else
			message	= 'OK'
			result  = 0
		end
		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					id			= id
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.del_schedule') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('delRepeaterSchedule', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local result	= value['result']
		local	id			= value['id']

		local traceId = context['traceId']
		local message = 'OK'
		if (result ~= 0) then
			result = 0
		end
		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					id			= id
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.mod_schedule') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('updateRepeaterSchedule', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end
		local result	= value['result']
		local	id			= value['id']

		local traceId = context['traceId']
		local message = 'OK'
		if (result ~= 0) then
			result = 0
		end
		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= result,	
				msg				= message,
				result		= {
					id			= id
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.timers') then
		-- get
		-- {"timers" : {"id":"", "repeat":"", "startTime":"", ""timeZone":"", "enabled":""}}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getRepeaterTimer', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local	slist		= value['timer_list']

		local timers = {}
		for k, v in pairs(slist) do
			timers[#timers+1] = {}
			timers[#timers]['id']					= v['id']
			timers[#timers]['total']			= v['total']
			timers[#timers]['remain']			= v['remain']
			timers[#timers]['state']			= v['state']
			timers[#timers]['action']			= v['action']
		end

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					timers	= timers
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.add_timer') then
		-- get
		-- {"result":0, id":""}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('addRepeaterTimer', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local result  = value['result']
		local id			= value['id']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					id		  = id
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.del_timer') then
		-- get
		-- {"result":0, id":""}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('delRepeaterTimer', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local result  = value['result']
		local id			= value['id']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					id		  = id
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.mod_timer') then
		-- get
		-- {"result":0, id":""}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('updateRepeaterTimerState', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local result  = value['result']
		local id			= value['id']

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					id		  = id
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.repeater.childrens') then
		-- get
		-- {"childres" : [{"mac":""}, {"mac":""}]}
		
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)

		local fixedHeader, variableHeader, context, data = fetch_one_cmd('getRepeaterConnectedDevices', get_gateway_device_id(), nil)
		if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
			return {}
		end

		local code	  = 0
		local traceId = context['traceId']
		local message = 'OK'
		local childres = value['childrens']

		local connectedDeviceIdList = {}
		for k, v in pairs(slist) do
			connectedDeviceIdList[#connectedDeviceIdList+1] = {}
			connectedDeviceIdList[#connectedDeviceIdList]['mac']					= v['mac']
		end

		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					connectedDeviceIdList = connectedDeviceIdList
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
		return rspnull()
	elseif (attr == 'device.ota_upgrade') then
		-- {"ModelStr":"ZB-WaterSensor-D0000","zone":"ZB-WaterSensor-D0000","battery":100,"code":208}
		local traceId = value['uuid']
		local modelstr	= value['ModelStr']
		local version = value['version'] or ''
		local firmName = value['firmName'] or nil
		modelstr	= devstr_search_by_modelstr(modelstr)

		local code = value['code']
		local message = 'OK'
		if (code == 209) then
			code = 100
			message = '100%'
		elseif (code == 208) then
			code = 207
			message = 'Download OK'
		elseif (code == 0 or code == 98) then
			code = 208
			message = 'Upgrade OK'
		elseif (code == 110 or code == 108 or code == 109 or code == 199 or code == 104) then
			code = 255
			message = 'Upgrade Failed'
		elseif (code == 130)  then
			code = 255
			message = 'Device Deleted'
		elseif (code == 131) then
			code = 255
			message = 'Version Error'
		elseif (code == 132) then
			code = -53002051
			message = 'Low Power'
			local ret=  {
				fixedHeader			= nil,
				variableHeader	= nil,
				payload					= {
					traceId		= traceId,
					code			= code,	
					msg				= message,
					result		= {
					}
				}
			}
			local sret = json.encode(ret['payload'])
			my_print('sret is ' .. sret)
			mqtt_publish('response', sret, #sret)
			return rspnull()
		elseif (code == 133) then
			code = -53002052
			message = 'Device Offline'
			local ret=  {
				fixedHeader			= nil,
				variableHeader	= nil,
				payload					= {
					traceId		= traceId,
					code			= code,	
					msg				= message,
					result		= {
					}
				}
			}
			local sret = json.encode(ret['payload'])
			my_print('sret is ' .. sret)
			mqtt_publish('response', sret, #sret)
			return rspnull()
		elseif (code == 134) then
			code = 255
			message = 'File Format Error'
		elseif (code == 99) then
			code = 0
			message = 'OK'
			local ret=  {
				fixedHeader			= nil,
				variableHeader	= nil,
				payload					= {
					traceId		= traceId,
					code			= code,	
					msg				= message,
					result		= {
					}
				}
			}
			local sret = json.encode(ret['payload'])
			my_print('sret is ' .. sret)
			mqtt_publish('response', sret, #sret)
			return rspnull()
		elseif (code == 105) then
			code = -53002053
			message = 'Busy'
			local ret=  {
				fixedHeader			= nil,
				variableHeader	= nil,
				payload					= {
					traceId		= traceId,
					code			= code,	
					msg				= message,
					result		= {
					}
				}
			}
			local sret = json.encode(ret['payload'])
			my_print('sret is ' .. sret)
			mqtt_publish('response', sret, #sret)
			return rspnull()
		else
			return rspnull()
		end

		local up_firmName			= nil
		local up_firmVersion	= nil
		local pos = 0
		if (firmName ~= nil) then
			pos							= string.find(firmName, '&')
			up_firmName			= string.sub(firmName, 1, pos-1)
			up_firmVersion	= string.sub(firmName, pos+1,-1)
		end
		local ret = {
			fixedHeader			= nil,
			variableHeader	= nil,
			payload					= {
				context = {
					traceId				= traceId,
					requestTime		= time_ms(),
					requestMethod	= 'upgradeNotify',
					deviceId			= get_gateway_device_id(),
					deviceModel		= get_gateway_model(),
				},
				data = {
					upgradeState	= tostring(code),
					subDeviceId		= mac, 
					subDeviceModel= modelstr,
					firmVersion		= version,
					firmName			= up_firmName
				}
			}
		}

		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is ' .. sret)
		mqtt_publish('report', sret, #sret)
		return rspnull()
	else 
		my_print('not support attribute :' .. attr)
		return {}	
	end
end

local rpt_handlers = {
	reportAttribute		=  report_handler,
	registerReq				=  register_handler,
	cmdResult					=  cmd_result_hanlder,
}

function rpt_handler_search(type, from, attr, mac, id, value) 
	rpt_handler = rpt_handlers[type]
	if (rpt_handler == nil) then
		my_print('not support type: ' .. type .. ' message')
		return nil	
	end
	return rpt_handler
end

--: cmd handler implement-------------------------------------------------------------------
function keypad_set_code(data, traceId) 
	local subDeviceId		  = data['deviceId']
	local deviceModel		= data['deviceModel']
	local code					= data['code']
	local codeId				= data['codeId']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.keypad.set_pass',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					codeId	= codeId,
					code		= code
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end
function keypad_del_code(data, traceId) 
	local subDeviceId		  = data['deviceId']
	local deviceModel		= data['deviceModel']
	local codeId				= data['codeId']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.keypad.del_pass',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					codeId	= codeId,
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end
function keypad_get_code(data, traceId) 
	local subDeviceId		  = data['deviceId']
	local deviceModel		= data['deviceModel']
	local codeId				= data['codeId']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.keypad.qry_pass',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					codeId	= codeId,
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0

end
function keypad_get_info(data, traceId) 
	local subDeviceId		  = data['deviceId']
	local deviceModel		= data['deviceModel']
	local codeId				= data['codeId']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.keypad.qry_stat',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					codeId	= codeId,
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function keypad_set_arm_delaytime(data, traceId) 
	local changeModeDelayTime = data['changeModeDelayTime']
	local mode = data['mode'] or 1

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.arm_delay',
				ep				=  1,
				mac				=	 gwmac(),
				value			=  {
					delay = changeModeDelayTime,
					mode  = mode
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function siren_get_effected_devices(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.relative',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function siren_update_siren_effected_devices(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local deviceList	= data['deviceList']
	local relative_list = {}
	for k, v in pairs(deviceList) do
		relative_list[#relative_list+1] = {}
		relative_list[#relative_list]['mac']			= v['deviceId']
		relative_list[#relative_list]['ModelStr']	= v['deviceModel']
		relative_list[#relative_list]['active']		= 1
	end
	
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.relative',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					relative_list = relative_list
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end


function siren_set_volume(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local volume			= data['volume']
	
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.voice_level',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					value		= volume
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0

end

function siren_set_siren_sounding(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local sounding		= data['sounding']
	
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.voicelight.alarm',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					value		= sounding
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end



function siren_get_volume(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.voice_level',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function siren_get_auto_off(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.auto_off',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function siren_set_auto_off(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local autoOff			= data['autoOff']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.siren.auto_off',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					value		= autoOff
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_set_switch(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local on			= data['on']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.onoff',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					value		= on
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_get_schedule(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.schedules',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_add_schedule(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local repeat_			= data['repeat']
	local startTime		= data['startTime']
	local timeZone		= data['timeZone']
	local enabled			= data['enabled']
	local startAct		= data['startAct']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.add_schedule',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					startTime	= startTime,
					timeZone	= timeZone,
					enabled		= enabled,
					startAct	= startAct
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	ubus_cmd['data']['arguments']['value']['repeat'] = repeat_
	return ubus_cmd, 0
end

function repeater_mod_schedule(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local repeat_			= data['repeat']
	local startTime		= data['startTime']
	local timeZone		= data['timeZone']
	local enabled			= data['enabled']
	local startAct		= data['startAct']
	local id					= data['id']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.mod_schedule',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					startTime	= startTime,
					timeZone	= timeZone,
					enabled		= enabled,
					startAct	= startAct,
					id				= id
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	ubus_cmd['data']['arguments']['value']['repeat'] = repeat_
	return ubus_cmd, 0
end


function repeater_del_schedule(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local id					= data['id']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.del_schedule',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					id			= id
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

-- timer 
function repeater_get_timer(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.timers',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_add_timer(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local action			= data['action']
	local total				= data['total']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.add_timer',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					action	= action,
					total		= total
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_upt_timer(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local	state				= data['state']
	local id					= data['id']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.upt_timer',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					id				= id,
					state			= state
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_get_connectted_devices(data, traceId) 
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.childrens',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function repeater_del_timer(data, traceId)
	local deviceId		= data['deviceId']
	local deviceModel	= data['deviceMode']
	local id					= data['id']

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.repeater.del_timer',
				ep				=  1,
				mac				=	 deviceId,
				value			=  {
					id			= id
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},

		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end



function yale_lock_create_entry_code(data, traceId) 
	local subDeviceId		= data['subDeviceId']
	local entryCodeId		= data['entryCodeId']
	local entryCode			= data['entryCode']
	local codeType			= data['codeType']
	local startDateTime = data['startDateTime']
	local endDateTime		= data['endDateTime']

	local startTime			= data['startTime']
	local endTime				= data['endTime']
	-- local timeZone			= data['timeZone']
	local repeats				= data['repeats']

	if (startDateTime ~= nil and type(startDateTime) ~= 'int') then
		startDateTime = tonumber(startDateTime)
	end
	if (endDateTime ~= nil and type(endDateTime) ~= 'int') then
		endDateTime		= tonumber(endDateTime)
	end

	if (startTime ~= nil ) then
		startTime = tonumber(string.sub(startTime, 1, 2)) * 60 + tonumber(string.sub(startTime, 4, 5))
	end
	if (endTime ~= nil) then
		endTime		= tonumber(string.sub(endTime, 1, 2)) * 60   + tonumber(string.sub(endTime, 4, 5))
	end

	if (repeats ~= nil and startTime ~= nil) then
		startTime = repeats * 10000 + startTime
	end

	if (repeats ~= nil and endTime ~= nil) then
		endTime		= repeats * 10000 + endTime
	end


	local _passId				= tonumber(entryCodeId)
	local _passType			= 0
	local _passVal1			= tonumber(entryCode)
	local _suspend			= 0
	local _passVal2			= 0
	local _startTime		= 0
	local _endTime			= 0

	if (codeType == 1) then -- admin
		passId			= 0
		_startTime	= 0
		_endTime		= 0
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 2) then -- main, no master
		passId			= passId
		_startTime	= 0
		_endTime		= 0
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 3) then -- tmp, shcudule year
		passId			= passId
		_startTime	= startDateTime
		_endTime		= endDateTime
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 4) then -- tmp, schudule day
		passId			= passId
		_startTime	= startTime
		_endTime		= endTime
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	else
		my_print(' not suppot codeType : ' .. codeType)
		return {}, 0
	end

	--attribute =  'device.lock.yale.create_entry_code',
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.add_password',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					passId			= _passId,
					passType		= _passType,
					passVal1		= _passVal1,
					passVal2		= _passVal2,
					startTime		= _startTime,
					endTime			= _endTime,
					suspend			= _suspend
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function yale_lock_alter_entry_code(data, traceId) 
	local subDeviceId		= data['subDeviceId']
	local entryCodeId		= data['entryCodeId']
	local entryCode			= data['entryCode']
	local codeType			= data['codeType']
	local startDateTime = data['startDateTime']
	local endDateTime		= data['endDateTime']
	local startTime			= data['startTime']
	local endTime				= data['endTime']
	local timeZone			= data['timeZone']
	local repeats				= data['repeats']

	if (startDateTime ~= nil and type(startDateTime) ~= 'int') then
		startDateTime = tonumber(startDateTime)
	end
	if (endDateTime ~= nil and type(endDateTime) ~= 'int') then
		endDateTime		= tonumber(endDateTime)
	end

	if (startTime ~= nil ) then
		startTime = tonumber(string.sub(startTime, 1, 2)) * 60 + tonumber(string.sub(startTime, 4, 5))
	end
	if (endTime ~= nil) then
		endTime		= tonumber(string.sub(endTime, 1, 2)) * 60   + tonumber(string.sub(endTime, 4, 5))
	end

	if (repeats ~= nil and startTime ~= nil) then
		startTime = repeats * 10000 + startTime
	end

	if (repeats ~= nil and endTime ~= nil) then
		endTime		= repeats * 10000 + endTime
	end


	local _passId				= tonumber(entryCodeId)
	local _passType			= 0
	local _passVal1			= tonumber(entryCode)
	local suspend				= 0
	local _passVal2			= 0
	local _startTime		= 0
	local _endTime			= 0

	if (codeType == 1) then -- admin
		passId			= 0
		_startTime	= 0
		_endTime		= 0
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 2) then -- main, no master
		passId			= passId
		_startTime	= 0
		_endTime		= 0
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 3) then -- tmp, shcudule year
		passId			= passId
		_startTime	= startDateTime
		_endTime		= endDateTime
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	elseif (codeType == 4) then -- tmp, schudule day
		passId			= passId
		_startTime	= startTime
		_endTime		= endTime
		_passVal2		= string.len(entryCode)
		if (_passVal2 < 4 or _passVal2 > 6) then
			return {}, 0
		end
	else
		my_print(' not suppot codeType : ' .. codeType)
		return {}, 0
	end

	--attribute =  'device.lock.yale.alter_entry_code',
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.add_password',
				ep				= 1,
				mac				=	 subDeviceId,
				value			=  {
					passId			= _passId,
					passType		= _passType,
					passVal1		= _passVal1,
					passVal2		= _passVal2,
					startTime		= _startTime,
					endTime			= _endTime,
					suspend			= _suspend
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function yale_lock_set_entry_code_state(data, traceId) 
	local subDeviceId			= data['subDeviceId']
	local entryCodeId			= data['entryCodeId']
	local entryCodeState	= data['entryCodeState']

	local _passId				= tonumber(entryCodeId)
	local _passType			= 0
	local _passVal1			= -1
	-- 1 del, 2 disable, 3 enable
	local _suspend			= entryCodeState
	local _passVal2			= -1
	local _startTime		= -1
	local _endTime			= -1

	local ubus_cmd = {}
	--attribute =  'device.lock.yale.alter_entry_code',
	if (_suspend == 1) then
		ubus_cmd = {
			data =  {
				arguments =  {
					attribute =  'device.lock.del_password',
					ep				= 1,
					mac				=	 subDeviceId,
					value			=  {
						passId			= _passId,
						clearAll		= 0
					}
				},
				command			=  'setAttribute',
				id					=  traceId
			},
			deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
			from					= 'CLOUD',
			mac						= gwmac(),
			time					= time(),
			to						= 'NXP',
			type					= 'cmd'
		}
 else 
		if (_suspend == 2) then
			_suspend = 1
		elseif (_suspend == 3) then
			_suspend = 0
		else
			return {}, 0
		end
		
		ubus_cmd = {
			data =  {
				arguments =  {
					attribute =  'device.lock.mod_password',
					ep				= 1,
					mac				=	 subDeviceId,
					value			=  {
						passId			= _passId,
						passType		= _passType,
						passVal1		= _passVal1,
						passVal2		= _passVal2,
						startTime		= _startTime,
						endTime			= _endTime,
						suspend			= _suspend
					}
				},
				command			=  'setAttribute',
				id					=  traceId
			},
			deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
			from					= 'CLOUD',
			mac						= gwmac(),
			time					= time(),
			to						= 'NXP',
			type					= 'cmd'
		}
	end

	return ubus_cmd, 0
end

function yale_lock_set_state(data, traceId) 
	local subDeviceId = data['subDeviceId']
	local lockState		= data['lockState']
	-- 0 -> close
	-- 1 -> open
	if (lockState == nil) then
		lockState = data['controlType']
	end

	if (lockState == 0) then
		lockState = 1
	else
		lockState = 0
	end

	local _passId				= 65488
	local _passType			= 10
	local _passVal1			= 0
	local _suspend			= 0
	local _passVal2			= lockState
	local _startTime		= 0
	local _endTime			= 0

	--attribute =  'device.lock.yale.set_state',
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.add_password',
				ep				=  1,
				mac				=	 subDeviceId,
				value			=  {
					passId			= _passId,
					passType		= _passType,
					passVal1		= _passVal1,
					passVal2		= _passVal2,
					startTime		= _startTime,
					endTime			= _endTime,
					suspend			= _suspend
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function yale_lock_set_auto_time_insec(data, traceId) 
	local subDeviceId = data['subDeviceId']
	local autoLockTimeInSec = data['autoLockTimeInSec']
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.set_auto_time_insec',
				ep				= 1,
				mac				=	 subDeviceId,
				value			= {
					time		= autoLockTimeInSec
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end


function yale_lock_clr_entry_code(data, traceId)
	local subDeviceId = data['subDeviceId']
	ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.del_password',
				ep				= 1,
				mac				=	 subDeviceId,
				value			=  {
					passId			= _passId,
					clearAll		= 1
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function yale_lock_get_one_touch_lock(data, traceId)
	local subDeviceId = data['subDeviceId']
	ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.one_touch_lock',
				ep				= 1,
				mac				=	 subDeviceId,
				value			=  {
					mac = subDeviceId
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function yale_lock_set_one_touch_lock(data, traceId)
	local subDeviceId = data['subDeviceId']
	local enable			= data['enable']
	ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.lock.one_touch_lock',
				ep				= 1,
				mac				=	 subDeviceId,
				value			=  {
					mac = subDeviceId,
					enable = enable
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd, 0
end

function set_device_config_mode_led_expired_func_2(arg) 
	my_print('close ap')

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		if (fs.access('/etc/cbtest')) then
			my_print('cbtest mode, dont close ap')
		else
			luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
			luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
		end
	else
		os.execute('uci set /etc/config/wireless.ap.hidden=1;uci commit;wifi reload wlan1&')
	end
end

function set_device_config_mode_led_expired_func_1(arg) 
	--my_print('close ap')

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		if (fs.access('/etc/cbtest')) then
			--my_print('cbtest mode, dont close ap')
		else
			luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
			luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
			luci.util.exec('/etc/init.d/dropbear stop;');
			luci.util.exec('/etc/init.d/dnsmasq stop;')
			luci.util.exec('/etc/init.d/uhttpd stop;')
			luci.util.exec('/etc/init.d/peripher-test stop;')
			luci.util.exec('/etc/init.d/device-manage stop;')
			luci.util.exec('/etc/init.d/appmged stop;')
		end
	else
		os.execute('uci set /etc/config/wireless.ap.hidden=1;uci commit;wifi reload wlan1&')
	end

	cache_schedule_add('setDeviceConfigModeLedExpiredTask_2', 10, 'set_device_config_mode_led_expired_func_2', 0)
end
function set_device_config_mode_led_expired_func(arg) 
	my_print('close config mode')
	luci.util.exec('mkdir -p /etc/config/dusun/mqtt-gw; echo 0 > /etc/config/dusun/mqtt-gw/configMode')

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('[ -e /tmp/._mqttconnect ] && gpio l 61 0 4000 0 1 4000 || gpio l 61 4000 0 1 0 4000')
		--luci.util.exec('gpio l 62 4000 0 1 0 4000')
	else
		luci.util.exec('echo none > /sys/class/leds/errled/trigger; echo 0 > /sys/class/leds/errled/brightness')
	end

	cache_schedule_add('setDeviceConfigModeLedExpiredTask_1', 10*60, 'set_device_config_mode_led_expired_func_1', 0)
end

function set_device_config_mode_0(data) 
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')

	--[[
	if (mtksdk == '1') then
		luci.util.exec('gpio l 61 10 10 4000 0 4000')
		--luci.util.exec('gpio l 62 10 10 4000 0 4000')
	else
		luci.util.exec('echo timer > /sys/class/leds/errled/trigger; echo 900 > /sys/class/leds/errled/delay_off; echo 100 > /sys/class/leds/errled/delay_on;')
	end
	--]]

	if (mtksdk == '1') then
		luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=0;uci commit /etc/config/wireless')
		luci.util.exec('iwpriv ra0 set HideSSID=0; wifi;')
		luci.util.exec('/etc/init.d/dropbear enable; /etc/init.d/dropbear start;')
		luci.util.exec('/etc/init.d/dnsmasq enable; /etc/init.d/dnsmasq start;')
		luci.util.exec('/etc/init.d/uhttpd enable; /etc/init.d/uhttpd start;')
		luci.util.exec('/etc/init.d/device-manage enable; /etc/init.d/device-manage start;')
		luci.util.exec('/etc/init.d/appmged enable; /etc/init.d/appmged start;')
		luci.util.exec('/etc/init.d/peripher-test enable; /etc/init.d/peripher-test start;')
	end
	
	local configType = 1
	luci.util.exec('mkdir -p /etc/config/dusun/mqtt-gw; echo ' .. configType .. ' > /etc/config/dusun/mqtt-gw/configMode')

	--cache_schedule_add('setDeviceConfigModeLedExpiredTask', 60, 'set_device_config_mode_led_expired_func', 0)
	cache_schedule_add('setDeviceConfigModeLedExpiredTask', 2*60, 'set_device_config_mode_led_expired_func', 0)
end
function set_device_config_mode(data) 
	-- TODO:
	-- 1->Apn, 2->ble
	--
	-- I thing there is no need to do any thing, whenever, 
	-- The gateway can be config wifi if it receive any config command
	--
	local configType = data['configType']
	if (configType ~= 1) then
		return 0 
	end

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 61 10 10 4000 0 4000')
		--luci.util.exec('gpio l 62 10 10 4000 0 4000')
	else
		luci.util.exec('echo timer > /sys/class/leds/errled/trigger; echo 900 > /sys/class/leds/errled/delay_off; echo 100 > /sys/class/leds/errled/delay_on;')
	end


	cache_schedule_del('setDeviceConfigModeLedExpiredTask_0')
	cache_schedule_del('setDeviceConfigModeLedExpiredTask')
	cache_schedule_del('setDeviceConfigModeLedExpiredTask_1')
	cache_schedule_del('setDeviceConfigModeLedExpiredTask_2')

	cache_schedule_add('setDeviceConfigModeLedExpiredTask_0', 3, 'set_device_config_mode_0', 0)
	return 0;
end

function upgrade_gateway_exit_func(arg) 
		luci.util.exec('rm -rf /tmp/._mqtt-gw_cache')
		os.exit()
		os.exit()
end

function upgrade_gateway_download_func(data)
	local upgradeUrl	= data['upgradeUrl']
	local firmVersion	= data['firmVersion']
	local md5sum			= data['md5sum']
	local usetime			= data['usetime']
	local traceId			= data['traceId']

	my_print(json.encode(data))

	usetime = usetime +5
	data['usetime']		= usetime
	local code = 255

	my_print(' downloading... ' .. usetime)
	local md5sum_download = string.gsub(luci.util.exec("md5sum /tmp/cbup.tar.gz | awk '{print $1}'"), "%s+", "")
	my_print('downloadmd5..:' .. md5sum_download .. ', need:' .. md5sum)
	if (md5sum_download ~= md5sum) then
		if (usetime > 7200) then
			my_print('uset time > 7200')
			code = 255
			luci.util.exec('killall wget')
		else
			my_print('use time < 7200, continue to wait...')
			local wgetcnt = luci.util.exec('ps | grep wget | grep -v "grep" | wc -l | tr -d "\r\n"')
			if ( wgetcnt == '0' ) then
				local rc = luci.util.exec('(wget -t 0 -T 30 -c ' .. upgradeUrl .. ' -O /tmp/cbup.tar.gz; echo -n $?)&')
			end
			cache_schedule_add('upgrade_gateway_download', 5, 'upgrade_gateway_download_func', data)
			return
		end
	else
		my_print('download ok!')
		code = 207
	end	

	local ret = {
		fixedHeader			= {},
		variableHeader	= {},
		payload					= {
			context = {
				traceId				= traceId,
				requestTime		= time_ms(),
				requestMethod	= 'upgradeNotify',
				deviceId			= get_gateway_device_id(),
				deviceModel		= get_gateway_model()
			},
			data = {
				upgradeState	= tostring(code),
				firmVersion   = firmVersion
			}
		}
	}
	
	if (code == 207) then
		ret['payload']['data']['upgradeState'] = tostring(100)
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('report', sret, #sret)
		ret['payload']['data']['upgradeState'] = tostring(207)
	end

	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('report', sret, #sret)

	if (code ~= 207) then
		luci.util.exec('rm -rf /tmp/cbup.tar.gz')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway download failed!')
		return 
	end

	my_print('md5sum ...')
	local md5sum_download = string.gsub(luci.util.exec("md5sum /tmp/cbup.tar.gz | awk '{print $1}'"), "%s+", "")
	if (md5sum_download ~= md5sum) then
		luci.util.exec('rm -rf /tmp/cbup.tar.gz')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway md5sum failed, dn:' .. md5sum_download .. ',cmd:' .. md5sum)
		return
	end

	my_print('backup...')
	-- backup
	
	--luci.util.exec('tar zcvf /etc/config/dusun/mqtt-gw/backup.tar.gz /usr/bin/mqtt-gw /etc/config/mqtt-gw /etc/config/dusun/mqtt-gw')
	luci.util.exec('tar zcvf /tmp/backup.tar.gz /usr/bin/mqtt-gw /etc/config/mqtt-gw /etc/config/dusun/mqtt-gw')

	my_print('xtar...')
	rc = luci.util.exec('tar zxvf /tmp/cbup.tar.gz -C / 2>&1 1>/dev/null; echo -n $?')
	if ( rc ~= '0' ) then
		--luci.util.exec('rm -rf /tmp/cbup.tar.gz; tar zxvf /etc/config/dusun/mqtt-gw/backup.tar.gz -C /; /etc/init.d/mqtt-gw start')
		luci.util.exec('(sleep 5;rm -rf /tmp/cbup.tar.gz; tar zxvf /tmp/backup.tar.gz -C /; /etc/init.d/mqtt-gw restart)&')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway uptar failed!')
		--os.exit()
		cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
		return
	end

	my_print('cbup.sh...')
	rc = luci.util.exec('[ -e /usr/bin/cbup.sh ] && echo -n 0 || echo -n 1')
	if (rc ~= '0') then
		luci.util.exec('(sleep 5; rm -rf /tmp/cbup.tar.gz; tar zxvf /tmp/backup.tar.gz -C /; /etc/init.d/mqtt-gw restart)&')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end
		my_print('upgate gateway cbup.sh failed!')
		--os.exit()
		cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
		return
	end

	ret['payload']['data']['upgradeState'] = tostring(208)
	sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('report', sret, #sret)
	local uprec_fp = io.open('/etc/config/dusun/mqtt-gw/up_record', 'w')
	if (uprec_fp ~= nil) then
		uprec_fp:write(sret)
		uprec_fp:close()
	end


	my_print('do cbup.sh...')
	luci.util.exec('rm -rf /tmp/cbup.tar.gz')
	luci.util.exec('rm -rf /tmp/backup.tar.gz')


	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 4000 0 1 0 4000')
	else
		luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
	end

	luci.util.exec('(sleep 5; chmod 777 /usr/bin/cbup.sh; /usr/bin/cbup.sh; rm -rf /usr/bin/cbup.sh)&')
	--luci.util.exec('sleep 8;');
	--os.exit()
	cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
	--]]

	
end
function upgrade_gateway_func(data)
	local upgradeUrl	= data['upgradeUrl']
	local firmVersion	= data['firmVersion']
	local md5sum			= data['md5sum']

	local fixedHeader, variableHeader, context, data = fetch_one_cmd('deviceUpgrade', get_gateway_device_id(), nil)
	if (fixedHeader == nil or variableHeader == nil or context== nil or data == nil) then
		my_print('no upgrade command cache')
		return
	end

	
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 10 10 4000 0 4000')
	else
		luci.util.exec('echo timer > /sys/class/leds/pwrled/trigger; echo 500 > /sys/class/leds/pwrled/delay_off; echo 500 > /sys/class/leds/pwrled/delay_on;')
	end
	

	my_print('download ...')
	--local rc = luci.util.exec('rm -rf /tmp/cbup.tar.gz 2>&1 1>/dev/null; wget -T 30 ' .. upgradeUrl .. ' -O /tmp/cbup.tar.gz; echo -n $?')
	--local rc = luci.util.exec('(rm -rf /tmp/cbup.tar.gz 2>&1 1>/dev/null; wget -T 30 ' .. upgradeUrl .. ' -O /tmp/cbup.tar.gz; echo -n $?)&')
	local rc = luci.util.exec('(killall wget;rm -rf /tmp/cbup.tar.gz 2>&1 1>/dev/null; wget -t 0 -T 30 -c ' .. upgradeUrl .. ' -O /tmp/cbup.tar.gz; echo -n $?)&')
	local code= 207
	if ( rc ~= '0' ) then
		-- down filed
		code =  255
	end

	data['usetime'] = 0
	data['traceId'] = context['traceId']
	cache_schedule_add('upgrade_gateway_download', 5, 'upgrade_gateway_download_func', data)

	return

	--[[
	local traceId = context['traceId']

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			context = {
				traceId				= traceId,
				requestTime		= time_ms(),
				requestMethod	= 'upgradeNotify',
				deviceId			= get_gateway_device_id(),
				deviceModel		= get_model(),
			},
			data = {
				upgradeState	= tostring(code)
			}
		}
	}
	
	if (code == 207) then
		ret['payload']['data']['upgradeState'] = 100
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('report', sret, #sret)
		ret['payload']['data']['upgradeState'] = 207
	end

	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('report', sret, #sret)

	if (code ~= 207) then
		luci.util.exec('rm -rf /tmp/cbup.tar.gz')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway download failed!')
		return 
	end

	my_print('md5sum ...')
	local md5sum_download = string.gsub(luci.util.exec("md5sum /tmp/cbup.tar.gz | awk '{print $1}'"), "%s+", "")
	if (md5sum_download ~= md5sum) then
		luci.util.exec('rm -rf /tmp/cbup.tar.gz')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway md5sum failed, dn:' .. md5sum_download .. ',cmd:' .. md5sum)
		return
	end

	my_print('backup...')
	-- backup
	
	--luci.util.exec('tar zcvf /etc/config/dusun/mqtt-gw/backup.tar.gz /usr/bin/mqtt-gw /etc/config/mqtt-gw /etc/config/dusun/mqtt-gw')
	luci.util.exec('tar zcvf /tmp/backup.tar.gz /usr/bin/mqtt-gw /etc/config/mqtt-gw /etc/config/dusun/mqtt-gw')

	my_print('xtar...')
	rc = luci.util.exec('tar zxvf /tmp/cbup.tar.gz -C / 2>&1 1>/dev/null; echo -n $?')
	if ( rc ~= '0' ) then
		--luci.util.exec('rm -rf /tmp/cbup.tar.gz; tar zxvf /etc/config/dusun/mqtt-gw/backup.tar.gz -C /; /etc/init.d/mqtt-gw start')
		luci.util.exec('(sleep 5;rm -rf /tmp/cbup.tar.gz; tar zxvf /tmp/backup.tar.gz -C /; /etc/init.d/mqtt-gw restart)&')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end

		my_print('upgate gateway uptar failed!')
		--os.exit()
		cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
		return
	end

	my_print('cbup.sh...')
	rc = luci.util.exec('[ -e /usr/bin/cbup.sh ] && echo -n 0 || echo -n 1')
	if (rc ~= '0') then
		luci.util.exec('(sleep 5; rm -rf /tmp/cbup.tar.gz; tar zxvf /tmp/backup.tar.gz -C /; /etc/init.d/mqtt-gw restart)&')

		local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
		if (mtksdk == '1') then
			luci.util.exec('gpio l 62 4000 0 1 0 4000')
		else
			luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
		end
		my_print('upgate gateway cbup.sh failed!')
		--os.exit()
		cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
		return
	end

	ret['payload']['data']['upgradeState'] = 208
	sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('report', sret, #sret)


	my_print('do cbup.sh...')
	luci.util.exec('rm -rf /tmp/cbup.tar.gz')
	luci.util.exec('rm -rf /tmp/backup.tar.gz')


	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 4000 0 1 0 4000')
	else
		luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness;')
	end

	luci.util.exec('(sleep 5; chmod 777 /usr/bin/cbup.sh; /usr/bin/cbup.sh; rm -rf /usr/bin/cbup.sh)&')
	--luci.util.exec('sleep 8;');
	--os.exit()
	cache_schedule_add('upgrade_gateway_exit', 8, 'upgrade_gateway_exit_func', 0)
	--]]
end

function upgrade_device(data, traceId) 
	local subDeviceId	= data['subDeviceId']
	local subDeviceModel	= data['subDeviceModel']
	local upgradeUrl	= data['upgradeUrl']
	local firmVersion = data['firmVersion']

	local firmName = nil
	if (data['firmInfo'] ~= nil and data['firmInfo']['firmName'] ~= nil and data['firmInfo']['firmVersion'] ~= nil) then
		firmName	= '' .. data['firmInfo']['firmName'] .. '&' .. data['firmInfo']['firmVersion']
	end
	if (data['firmName'] ~= nil) then
		firmName = data['firmName'] .. '&' .. firmVersion
	end

	local md5sum			= data['md5sum']
	if (upgradeUrl == nil or firmVersion == nil or md5sum == nil) then
		return 101
	end
	if (subDeviceId == nil) then
		return 101
	end

	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.ota_upgrade',
				ep				=  1,
				mac				=	 subDeviceId,
				value			= {
					url					= upgradeUrl,
					md5sum			= md5sum,
					firmVersion	= firmVersion,
					firmName		= firmName
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= traceId,
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	ubus_send('NXP', ubus_cmd)
	return 0
end

function upgrade_gateway(data)
	local upgradeUrl	= data['upgradeUrl']
	local firmVersion	= data['firmVersion']
	local md5sum			= data['md5sum']
	if (upgradeUrl == nil or firmVersion == nil or md5sum == nil) then
		return 101
	end

	-- upgrade firmware
	-- TODO:
	--[[
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'gateway.upgrade_firmware',
				ep				= 1,
				mac				=	 gwmac(),
				value			= {
					url			= upgradeUrl,
					md5sum	= md5sum
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'GATEWAY',
		type					= 'cmd'
	}
	ubus_send('GATEWAY', ubus_cmd)
	--]]
	
	-- upgrade app
	
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 1 1 0 0 0')
	else
		luci.util.exec('echo oneshot > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/shot')
	end
	
	

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 1 1 0 0 0')
	else
		local zigbee_led_mode = luci.util.exec('cat /sys/class/leds/zigbee/trigger | awk \'{printf "%s",$3}\'')
		if (zigbee_led_mode ~= '[timer]') then
			luci.util.exec('echo oneshot > /sys/class/leds/zigbee/trigger; echo 1 > /sys/class/leds/zigbee/shot')
		end
	end

	cache_schedule_add('upgrade_gateway', 1, 'upgrade_gateway_func', data)
	
	return 0
end

function upgrade_gateway_firmware(data, traceId)
	local upgradeUrl	= data['upgradeUrl']
	local firmVersion	= data['firmVersion']
	local md5sum			= data['md5sum']
	if (upgradeUrl == nil or firmVersion == nil or md5sum == nil) then
		return 101
	end

	-- upgrade firmware
	-- TODO:
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'gateway.upgrade_firmware',
				ep				= 1,
				mac				=	 gwmac(),
				value			= {
					url			= upgradeUrl,
					md5sum	= md5sum
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'GATEWAY',
		type					= 'cmd'
	}
	ubus_send('GATEWAY', ubus_cmd)
	
	-- upgrade app
	--[[
	
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 1 1 0 0 0')
	else
		luci.util.exec('echo oneshot > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/shot')
	end
	
	

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 1 1 0 0 0')
	else
		local zigbee_led_mode = luci.util.exec('cat /sys/class/leds/zigbee/trigger | awk \'{printf "%s",$3}\'')
		if (zigbee_led_mode ~= '[timer]') then
			luci.util.exec('echo oneshot > /sys/class/leds/zigbee/trigger; echo 1 > /sys/class/leds/zigbee/shot')
		end
	end

	cache_schedule_add('upgrade_gateway', 1, 'upgrade_gateway_func', data)
	--]]
	
	return 0
end



function batch_delete_sub_device(data)
	local subDeviceIdList = data['subDeviceIdList']
	if (subDeviceIdList == nil) then
		return -1
	end
	
	for i=1,#subDeviceIdList do
		subDeviceId = subDeviceIdList[i]
		my_print('delete subdevice ' .. subDeviceId)
		--TODO:
		os.execute('dusun_ucmd.sh remove ' .. subDeviceId)
	end
	return 0
end

function add_sub_device_timeout_func(arg) 
	local traceId			= arg['traceId']
	local subDeviceId	= arg['subDeviceId']
	local fixedHeader, variableHeader, context, data = fetch_one_cmd('addSubDevice', get_gateway_device_id(), subDeviceId)
	
	local message = 'TimeOut'
	local code = -50382003

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
				subDeviceId		= subDeviceId,
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)
end
function add_sub_device(traceId, data)
	local subDeviceId = data['subDeviceId']
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'mod.add_device',
				ep				= 1,
				mac				=	 subDeviceId,
				value			= {
					mac			= subDeviceId,
					type		= '0000',
				}
			},
			command			=  'setAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	--cache_schedule_add('add_subdevice_' .. subDeviceId, 75, 'add_sub_device_timeout_func', {traceId = traceId, subDeviceId = subDeviceId})
	return ubus_cmd

	--[[
	return 0, {
		subDeviceId = '01c6e408d4232a422882c2919e390f55b3',
		deviceModel	= 'etekcity',
		sn					=	'5465454654646',
		firmVersion	= '1.0.1',
		online			= 1,
		state				= 1,
		tamper			= 0,
		voltage			= 0,
		battery			= 0
 }
	--]]
end

--[[
function reset_gateway_expired_func_1(arg) 
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 4000 0 1 0 4000')
	else
		luci.util.exec('echo none > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/brightness')
	end
end
--]]
function reset_gateway_expired_func(arg) 
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	--[[
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 10 10 4000 0 4000')
	else
		luci.util.exec('echo timer > /sys/class/leds/pwrled/trigger; echo 500 > /sys/class/leds/pwrled/delay_off; echo 500 > /sys/class/leds/pwrled/delay_on;')
	end
	--cache_schedule_add('resetGatewayExpiredTask_1', 5, 'reset_gateway_expired_func_1', 0)
	--]]

	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 4000 0 1 0 4000')

		my_print('reset gateway wifi')
		luci.util.exec('uci set /etc/config/apcli.sta.ssid="______";uci set /etc/config/apcli.sta.password="______";uci commit /etc/config/apcli;');
		luci.util.exec('kill -16 `cat /tmp/apcli-apcli0.pid | tr -d "\r\n"`')
	end
end
function reset_gateway(data, traceId) 
	local resetType = data['resetType']


	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	--[[
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 1 1 0 0 0')
	else
		luci.util.exec('echo oneshot > /sys/class/leds/pwrled/trigger; echo 1 > /sys/class/leds/pwrled/shot')
	end

	if (mtksdk == '1') then
		luci.util.exec('gpio l 63 1 1 0 0 0')
	else
		local zigbee_led_mode = luci.util.exec('cat /sys/class/leds/zigbee/trigger | awk \'{printf "%s",$3}\'')
		if (zigbee_led_mode ~= '[timer]') then
		 luci.util.exec('echo oneshot > /sys/class/leds/zigbee/trigger; echo 1 > /sys/class/leds/zigbee/shot')
		end
	end
	--]]
	if (mtksdk == '1') then
		luci.util.exec('gpio l 62 10 10 4000 0 4000')
	end


	if (resetType == 5) then
		-- reset the wifi info
		--[[
		my_print('reset gateway wifi')
		luci.util.exec('uci set /etc/config/apcli.sta.ssid="______";uci set /etc/config/apcli.sta.password="______";uci commit /etc/config/apcli;');
		luci.util.exec('kill -16 `cat /tmp/apcli-apcli0.pid | tr -d "\r\n"`')
		--]]
	elseif (resetType == 10) then
		my_print('reset zigbee')
		local ubus_cmd = {
			data =  {
				arguments =  {
					attribute =  'device.clear_device',
					ep				=  1,
					mac				=	 gwmac(),
					value			=  {
						yum = yum
					}
				},
				command			=  'setAttribute',
				id					=  traceId
			},
			from					= 'CLOUD',
			mac						= gwmac(),
			time					= time(),
			to						= 'NXP',
			type					= 'cmd'
		}
		ubus_send('NXP', ubus_cmd)
	elseif (resetType == 80) then
		my_print('total reset ')
		--[[
		luci.util.exec('uci set /etc/config/apcli.sta.ssid="______";uci set /etc/config/apcli.sta.password="______";uci commit /etc/config/apcli;');
		luci.util.exec('kill -16 `cat /tmp/apcli-apcli0.pid | tr -d "\r\n"`')
		--]]
		my_print('reset zigbee')
		local ubus_cmd = {
			data =  {
				arguments =  {
					attribute =  'device.clear_device',
					ep				=  1,
					mac				=	 gwmac(),
					value			=  {
						yum = yum
					}
				},
				command			=  'setAttribute',
				id					=  traceId
			},
			from					= 'CLOUD',
			mac						= gwmac(),
			time					= time(),
			to						= 'NXP',
			type					= 'cmd'
		}
		ubus_send('NXP', ubus_cmd)
	else
		my_print('not support')
	end

	--cache_schedule_add('resetGatewayExpiredTask', 1, 'reset_gateway_expired_func', 0)
	cache_schedule_add('resetGatewayExpiredTask', 10, 'reset_gateway_expired_func', 0)

	return 0, resetType
end

function set_arm_mode(data)
	local old_armMode = get_gateway_arm_mode()

	local armMode			= data['armMode']
	set_gateway_arm_mode(armMode)

	if (armMode ~= old_armMode) then
		local ubus_msg = {
			to		= 'GATEWAY',
			from	= 'NXP',
			type	= 'reportAttribute',
			data	= {
				mac	= gwmac(),
				attribute		= 'gateway.state.change',
				value				= {
					mac = gwmac()
				}
			},
			time				= time(),
			deviceCode	= get_devcode(),
			mac					= gwmac()
		}
		ubus_send('GATEWAY', ubus_msg)

		local deviceArmMode  =armMode
		if (deviceArmMode == 2) then
			deviceArmMode = 3
		end
		local ubus_cmd = {
			data =  {
				arguments =  {
					attribute =  'device.arm_status',
					ep				=  1,
					mac				=	 gwmac(),
					value			=  {
						value		= deviceArmMode
					}
				},
				command			=  'setAttribute',
				id					=  traceId
			},
			from					= 'CLOUD',
			mac						= gwmac(),
			time					= time(),
			to						= 'NXP',
			type					= 'cmd'
		}
		ubus_send('NXP', ubus_cmd)
	end
	return 0
end

function get_sub_device_list(traceId)
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'mod.device_list',
				mac				=	 gwmac(),
			},
			command			=  'getAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd

	-- subDeviceId [yes]
	-- deviceModel [yes]
	-- sn					 [yes]
	-- firmVersion [yes]
	--[[
	return 0,  {{
		subDeviceId	= '01c6e408d4232a422882c2919e390f55b3',
		deviceModel	= 'etekcity',
		sn					= '5465454654646',
		firmVersion	= '1.0.1',
		online			= 1,
		state				= 1,
		tamper			= 0,
		voltage			= 0,
		battery			= 0,
	}, {
		subDeviceId	= '01c6e408d4232a422882c2919e390f55b3',
		deviceModel	= 'etekcity',
		sn					= '5465454654646',
		firmVersion	= '1.1.15',
		online			= 1,
		state				= 0,
		tamper			= 0,
		voltage			= 0
		battery			= 0,
	}}
	--]]
end

function get_subdevice_signal(data, traceId)
	local subDeviceId = data['subDeviceId']
	local ubus_cmd = {
		data =  {
			arguments =  {
				attribute =  'device.signal',
				mac				=	 subDeviceId,
				value			= {
					mac			= subDeviceId
				}
			},
			command			=  'getAttribute',
			id					=  traceId
		},
		deviceCode		= 'db4a7b00-08f4-49e3-9ace-ac59ce035737',
		from					= 'CLOUD',
		mac						= gwmac(),
		time					= time(),
		to						= 'NXP',
		type					= 'cmd'
	}
	return ubus_cmd
end

function password_mask(psw) 
	local iS     = string.byte(" ", 1)
	local iA     = string.byte("A", 1)
	local iZ     = string.byte("Z", 1)
	local pswMask = ''

	for i=1,string.len(psw) do
		local ix     = string.byte(psw, i)
		if (i == 1) then
			if (ix >= iA and ix <= iZ) then
				pswMask = pswMask .. '_'
			elseif (ix == iS) then
				pswMask = pswMask .. ' '
			else
				pswMask = pswMask .. '*'
			end
		else
			if (ix == iS) then
				pswMask = pswMask .. ' '
			else
				pswMask = pswMask .. '*'
			end
		end
	end
	return pswMask
end

function set_wifi_ap_expired_func(arg)
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	local rc = 0
	if (mtksdk == '1') then
		rc = luci.util.exec('ifconfig apcli0 | grep "inet addr" 2> /dev/null | wc -l | tr -d "\r\n"')
	else
		rc = luci.util.exec('ifconfig wlan0 | grep "inet addr" 2> /dev/null | wc -l | tr -d "\r\n"')
	end

	local timesec = arg

	local ssid		= luci.util.exec('uci get /etc/config/apcli.sta.ssid | tr -d "\r\n"')
	local pass		= luci.util.exec('uci get /etc/config/apcli.sta.password | tr -d "\r\n"')
	local pswMd5	= luci.util.exec('echo ' .. pass .. ' -n | md5sum | awk \'{printf "%s",$1}\'')
	local pswMask = password_mask(pass)

	--my_print('timesec :'  .. timesec)
	if (rc == '0') then
		if (timesec < 120) then
			cache_schedule_add('setWifiApExpiredTask', 5, 'set_wifi_ap_expired_func', timesec+5)
		else
			local configId=luci.util.exec('cat /tmp/._set_wifi_ap_configId | tr -d "\r\n"');

			local Protocol			= '01'
			local ServiceType		= '01'
			local SequenceId		= '0001'
			-- bit[7~4]: security, 0000 none, 0001 RSA1024/AES128/CBC-PKCS1Padding
			-- bit[3~0]: Version, 0010
			local Flag					= '12'
			local OpCode				= '8014'
			local StatusCode		= 0
			local code = 0

			local ret = {
				Protocol		= Protocol,
				ServiceType	= ServiceType,
				SequenceId	= SequenceId,
				Length			= 0,
				Flag				= Flag,
				OpCode			= OpCode,
				StatusCode	= code,
				Payload			= {
					configId	= configId,
					step			= 'CONNECT_WIFI',
					code			= -50012009,
					msg				= 'PasswordError',
					debugInfo = {
						ssid		= ssid,
						pswMd5	= pswMd5,
						pswMask	= pswMask
					}
				}
			}
			ret.Length			= #json.encode(ret.Payload)
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			tcp_cfg_out('c', sret, #sret)

			luci.util.exec('uci set /etc/config/apcli.sta.ssid="______";uci set /etc/config/apcli.sta.password="______";uci commit /etc/config/apcli;');
			luci.util.exec('rm -rf /tmp/._set_wifi_ap_configId')
			luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
			luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
		end
	else
		local configId=luci.util.exec('cat /tmp/._set_wifi_ap_configId | tr -d "\r\n"');

		local wifi_ip='0.0.0.0'
		local wifi_dns='0.0.0.0'
		if (mtksdk == '1') then
			wifi_ip		= luci.util.exec('ifconfig apcli0 | grep "inet addr" | xargs | cut -d ":" -f 2 | cut -d " " -f 1 | tr -d "\r\n"')
			wifi_dns	= luci.util.exec('cat /tmp/._wan1.gw | tr -d "\r\n"')
		else
			wifi_ip = luci.util.exec('ifconfig eth0.2 | grep "inet addr" | xargs | cut -d ":" -f 2 | cut -d " " -f 1 | tr -d "\r\n"')
			wifi_dns	= luci.util.exec('cat /tmp/._wan1.gw | tr -d "\r\n"')
		end
		local rssi=-60
		mqtt_reconnect(0)

		local Protocol			= '01'
		local ServiceType		= '01'
		local SequenceId		= '0001'
		-- bit[7~4]: security, 0000 none, 0001 RSA1024/AES128/CBC-PKCS1Padding
		-- bit[3~0]: Version, 0010
		local Flag					= '12'
		local OpCode				= '8014'
		local StatusCode		= 0
		local code = 0
		local msg = 'OK'

		local ret = {
			Protocol		= Protocol,
			ServiceType	= ServiceType,
			SequenceId	= SequenceId,
			Length			= 0,
			Flag				= Flag,
			OpCode			= OpCode,
			StatusCode	= code,
			Payload			= {
				configId	= configId,
				step			= 'CONNECT_WIFI',
				code			= code,
				msg				= msg,
				debugInfo = {
						ssid		= ssid,
						pswMd5	= pswMd5,
						pswMask	= pswMask
				}
			}
		}
		ret.Length			= #json.encode(ret.Payload)
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		tcp_cfg_out('c', sret, #sret)

		local ret = {
			Protocol		= Protocol,
			ServiceType	= ServiceType,
			SequenceId	= SequenceId,
			Length			= 0,
			Flag				= Flag,
			OpCode			= OpCode,
			StatusCode	= code,
			Payload			= {
				configId	= configId,
				step			= 'CONNECT_SERVER',
				code			= code,
				msg				= msg,
				debugInfo = {
				}
			}
		}
		ret.Length			= #json.encode(ret.Payload)
		sret = json.encode(ret)
		my_print('sret is' .. sret)
		tcp_cfg_out('c', sret, #sret)



		local ret = {
			Protocol		= Protocol,
			ServiceType	= ServiceType,
			SequenceId	= SequenceId,
			Length			= 0,
			Flag				= Flag,
			OpCode			= OpCode,
			StatusCode	= code,
			Payload			= {
				configId	= configId,
				step			= 'GET_IP',
				code			= code,
				msg				= msg,
				debugInfo = {
					ip = wifi_ip
				}
			}
		}
		ret.Length			= #json.encode(ret.Payload)
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		tcp_cfg_out('c', sret, #sret)


		ret = {
			Protocol		= Protocol,
			ServiceType	= ServiceType,
			SequenceId	= SequenceId,
			Length			= 0,
			Flag				= Flag,
			OpCode			= OpCode,
			StatusCode	= code,
			Payload			= {
				configId	= configId,
				step			= 'DNS_RESOLVE',
				code			= code,
				msg				= msg,
				debugInfo = {
					serverip		= wifi_dns,
					routerRssi	= rssi
				}
			}
		}
		ret.Length			= #json.encode(ret.Payload)
		sret = json.encode(ret)
		my_print('sret is' .. sret)
		tcp_cfg_out('c', sret, #sret)

		luci.util.exec('rm -rf /tmp/._set_wifi_ap_configId')
		luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
		luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
	end
end

function set_wifi_expired_func(arg)

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	local rc = 0
	if (mtksdk == '1') then
		rc = luci.util.exec('ifconfig apcli0 | grep "inet addr" 2> /dev/null | wc -l | tr -d "\r\n"')
	else
		rc = luci.util.exec('ifconfig wlan0 | grep "inet addr" 2> /dev/null | wc -l | tr -d "\r\n"')
	end

	local ssid		= luci.util.exec('uci get /etc/config/apcli.sta.ssid | tr -d "\r\n"')
	local pass		= luci.util.exec('uci get /etc/config/apcli.sta.password | tr -d "\r\n"')
	local pswMd5	= luci.util.exec('echo ' .. pass .. ' -n | md5sum | awk \'{printf "%s",$1}\'')
	local pswMask = password_mask(pass)

	local timesec = arg

	--my_print('timesec :'  .. timesec)
	if (rc == '0') then
		if (timesec < 120) then
			cache_schedule_add('setWifiExpiredTask', 5, 'set_wifi_expired_func', timesec+5)
		else
			local configId=luci.util.exec('cat /tmp/._set_wifi_configId | tr -d "\r\n"');
			local ret = {
				context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'configStateNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model()
				},
				data = {
					configId				= configId,
					step						= 'CONNECT_WIFI',
					code						= -50012009, 
					msg							= 'PasswordError',
					debugInfo = {
						ssid		= ssid,
						pswMd5	= pswMd5,
						pswMask	= pswMask
					}
				}
			}
			local sret = json.encode(ret)
			my_print('sret is' .. sret)
			mqtt_publish('report', sret, #sret)

			luci.util.exec('uci set /etc/config/apcli.sta.ssid="______";uci set /etc/config/apcli.sta.password="______";uci commit /etc/config/apcli;');
			luci.util.exec('rm -rf /tmp/._set_wifi_configId')
			luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
			luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
		end
	else
		local wifi_ip='0.0.0.0'
		local wifi_dns='0.0.0.0'
		if (mtksdk == '1') then
			wifi_ip		= luci.util.exec('ifconfig apcli0 | grep "inet addr" | xargs | cut -d ":" -f 2 | cut -d " " -f 1 | tr -d "\r\n"')
			wifi_dns	= luci.util.exec('cat /tmp/._wan1.gw | tr -d "\r\n"')
		else
			wifi_ip = luci.util.exec('ifconfig eth0.2 | grep "inet addr" | xargs | cut -d ":" -f 2 | cut -d " " -f 1 | tr -d "\r\n"')
			wifi_dns	= luci.util.exec('cat /tmp/._wan1.gw | tr -d "\r\n"')
		end
		local rssi=-60

		mqtt_reconnect(0)

		local configId=luci.util.exec('cat /tmp/._set_wifi_configId | tr -d "\r\n"');
		local code = 0
		local msg = 'OK'
		local ret = {
			context = {
				traceId					= get_uuid(),
				requestTime			= time_ms(),
				requestMethod		= 'configStateNotify',
				deviceId				= get_gateway_device_id(),
				deviceModel			= get_gateway_model()
			},
			data = {
				configId				= configId,
				step						= 'CONNECT_WIFI',
				code						= code, 
				msg							= msg,
				debugInfo = {
						ssid		= ssid,
						pswMd5	= pswMd5,
						pswMask	= pswMask
				}
			}
		}
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)

		ret.data = {
			configId				= configId,
			step						= 'CONNECT_SERVER',
			code						= code, 
			msg							= msg,
			debugInfo				= {
			}
		}
		sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)



		ret.data = {
			configId				= configId,
			step						= 'GET_IP',
			code						= code, 
			msg							= msg,
			debugInfo				= {
				ip		= wifi_ip
			}
		}
		sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)

		ret.data = {
			configId				= configId,
			step						= 'DNS_RESOLVE',
			code						= code, 
			msg							= msg,
			debugInfo				= {
				serverip		= wifi_dns,
				routerRssi	= rssi
			}
		}
		sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)

		luci.util.exec('rm -rf /tmp/._set_wifi_configId')
		luci.util.exec('uci set /etc/config/wireless.@wifi-iface[0].hidden=1;uci commit /etc/config/wireless')
		luci.util.exec('iwpriv ra0 set HideSSID=1; ifconfig ra0 down;')
	end
end
function set_wifi_mtksdk(data, traceId)
	local ssid			= data['ssid'] or '______'
	local pwd				= data['pwd']	or '______'
	local chan			= data['chan']
	local configId	= data['configId'] or "noconfigId"

	if (ssid == '') then
		ssid = '______'
	end
	if (pwd == '') then
		pwd = '______'
	end
	
	local flag = 0
	local i = 0
	--luci.util.exec('ifconfig ra0 up')
	while i < 5 do
		my_print('--try ' .. i)
		--[[
		local essid = luci.util.exec('iwpriv ra0 set SiteSurvey="' .. ssid .. '"; sleep 3; iwpriv ra0 get_site_survey | awk \'{if ($2 == "' .. ssid .. '") print $2}\' | tr -d "\r\n"')
		if (essid == ssid) then
			flag = 1
			break
		end
		--]]
		local fret = iwpriv(ssid)
		if (fret == 0) then
			iwsave(ssid)
			flag = 1
			break;
		end
		
		i = i + 1
	end
	--luci.util.exec('ifconfig ra0 down')

	if (flag == 0) then
		local ret = {
			context = {
					traceId					= get_uuid(),
					requestTime			= time_ms(),
					requestMethod		= 'configStateNotify',
					deviceId				= get_gateway_device_id(),
					deviceModel			= get_gateway_model()
			},
			data = {
					configId				= configId,
					step						= 'CONNECT_WIFI',
					code						= -50012008, 
					msg							= 'NoSSID',
					debugInfo				= {
						ssid = wfssid
					}
			}
		}
		local sret = json.encode(ret)
		my_print('sret is' .. sret)
		mqtt_publish('report', sret, #sret)
		return 0
	end

	-- do set wifi
	print('setting ssid:' .. ssid .. ',pwd:' .. pwd .. ',chan:' .. chan)
	luci.util.exec('uci set /etc/config/apcli.sta.ssid="' .. ssid .. '";uci set /etc/config/apcli.sta.password="' .. pwd .. '";uci commit /etc/config/apcli;');

	--[[
	local ret = {
		context = {
			traceId					= get_uuid(),
			requestTime			= time_ms(),
			requestMethod		= 'configStateNotify',
			deviceId				= get_gateway_device_id(),
			deviceModel			= get_gateway_model()
		},
		data = {
			configId				= configId,
			step						= 'RECEIVE_CONFIG',
			code						= 0, 
			msg							= 'OK',
			debugInfo				= {}
		}
	}
	local sret = json.encode(ret)
	my_print('sret is' .. sret)
	mqtt_publish('report', sret, #sret)
	--]]
	
	--[[
	local __s = fs.readfile("/tmp/._wifilist")
	local wflist = json.decode(__s) or {}
	local wfssid = luci.util.exec('uci get /etc/config/apcli.sta.ssid | tr -d "\r\n"')
	local flag = 0
	for i=1,#wflist do
		if (wflist[i]['ssid'] ==  wfssid) then
			flag = 1
		end
	end
	--]]
	
	--luci.util.exec('echo ' .. traceId .. ' > /tmp/._set_wifi_configId')
	luci.util.exec('echo ' .. configId .. ' > /tmp/._set_wifi_configId')

	luci.util.exec('kill -16 `cat /tmp/apcli-apcli0.pid | tr -d "\r\n"`')
	cache_schedule_add('setWifiExpiredTask', 20, 'set_wifi_expired_func', 20)
	--luci.util.exec('sleep 3; wifi')
	return 0
end

function set_wifi(data, traceId)
	-- v0.0.2
	-- [[
	local wifiSSid	= data['wifiSsid']
	local wifiPwd		=	data['wifiPwd']	
	--]]
	
	-- v0.0.4
	local ssid			= data['ssid']
	local pwd				= data['pwd']
	local chan			= data['chan']
	local configId	= data['configId'] or "noconfigId"

	-- do set wifi
	print('setting ssid:' .. ssid .. ',pwd:' .. pwd .. ',chan:' .. chan)


	local etype = 'psk2'

	local xcnt = 3
	local find = 0
	for j=1,xcnt do 
		if (find == 0) then
			wfx = nil
			my_print('find is ' .. find)
			local wcode, wls = get_wifi_list_s() 
			for i=1,#wls do
				if (wls[i]['SSID'] == ssid) then
					wfx = wls[i]
				end
			end
			if (wfx ~= nil) then
				find = 1
				my_print('find is ' .. find)
				my_print(json.encode(wfx))
				if (wfx['TKIP'] == '1') then
					etype = 'psk2+tkip'
				end
			end
		end
	end
	my_print('etype is ' .. etype)
	

	local n = 0                                               
        local wn=""                                               
        uci:foreach("wireless", "wifi-iface",               
                function (s)                                                       
                        if n == 0 then                                             
                                wn = s['.name']                                    
                        end                                                        
                        n = n + 1                                                  
		end
	)
	uci:set('wireless', wn, 'ssid', ssid)
	--uci:set('wireless', wn, 'encryption', 'psk2+tkip')
	uci:set('wireless', wn, 'encryption', etype)
	uci:set('wireless', wn, 'key',	      pwd)
	uci:set('wireless', wn, 'network', 'wan1')
	uci:set('wireless', wn, 'disabled', '0')
	uci:set('wireless', wn, 'mode', 'sta')
  uci:commit("wireless")

  uci:set("network", "wan1", "interface")
  uci:set("network", "wan1", "proto", "dhcp")
  uci:commit("network")

  n=0
  local fn=""
  uci:foreach("firewall", "zone",
		function (s)
		 if n == 1 then
		  fn = s['.name']
     end
     n = n + 1
    end
  )
	local nks=uci:get_list("firewall", fn, "network")
  for k, v in pairs(nks) do
		if v == "wan1" then
		 	table.remove(nks, k)
    end
  end
  table.insert(nks, "wan1")
  -- luci.util.dumptable(nks)
  uci:set_list("firewall", fn, "network", nks)
  uci:commit("firewall")

  os.execute("/etc/init.d/firewall reload 2>1 1>/dev/null")
  os.execute("wifi reload wlan1")

	--configId	= traceId,
	local ubus_msg = {
			to		= 'GATEWAY',
			from	= 'NXP',
			type	= 'reportAttribute',
			data	= {
				mac	= gwmac(),
				attribute		= 'gateway.wifi.config.process',
				value				= {
					configId	= configId,
					step			= 'RECEIVE_CONFIG',
					code			= 0,
					msg				= 'OK',
					debugInfo = {
					}
				}
			},
			time				= time(),
			deviceCode	= get_devcode(),
			mac					= gwmac()
	}
	ubus_send('GATEWAY', ubus_msg)
		
	ubus_msg['data']['value']['step'] = 'CONNECT_WIFI'
	ubus_send('GATEWAY', ubus_msg)
	
	--luci.util.exec('echo ' .. traceId .. ' > /tmp/._set_wifi_configId')
	luci.util.exec('echo ' .. configId .. ' > /tmp/._set_wifi_configId')

	cache_schedule_add('setWifiExpiredTask', 20, 'set_wifi_expired_func', 20)
	return 0
end

function get_wifi_list_mtksdk() 
	--local __s = fs.readfile("/tmp/._wifilist")
	local __s = iwscan('')
	local wflist = json.decode(__s) or {}

	local wl = {}
	for i=1,#wflist do
		if (wflist[i]['ssid'] ~= ' ' and wflist[i]['ssid'] ~= '') then
			wl[#wl+1] = {}

			wl[#wl]['ssid'] = wflist[i]['ssid']
			--wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['Signal'])
			--wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['rssi'])
			wl[#wl]['rssi'] = quanity2rssi(tonumber(wflist[i]['rssi']))
			wl[#wl]['chan'] = tonumber(wflist[i]['Channel'])

			local auth_int = 0
			-- WPAPSK WPA2PSK		WPAPSKWPA2PSK
			local security = wflist['security'] 
			-- AES TKIP TKIPAES
			local crypto	 = wflist['crypto']
			
			if (security == 'NONE') then
				auth_int = 0
			elseif (security == 'WPAPSK') then
				auth_int = 2
			elseif (security == 'WPA2PSK') then
				auth_int = 3
			elseif (security == 'WPAPSKWPA2PSK') then
				auth_int = 4
			else
				auth_int = 5
			end

			wl[#wl]['auth'] = auth_int
		end
	end

	return 0, wl
end
function get_wifi_list_s() 
	local wfenable = uci:get('wireless', 'radio0', 'disabled')
	if (wfenable == '1') then
		uci:set('wireless', 'radio0', 'disabled', '0')
		uci:commit('wireless')
		os.execute('wifi&;sleep 8')
	end

	-- ps=luci.util.execi('iw dev wlan1 scan')
	local ps=luci.util.execi('iwlist wlan1 scanning')
	if not ps then
        return 0, {}
	else
        ps()
	end

	local wflist = {}
	for i in ps do
		local s,e = string.find(i, 'Cell.+Address:+')
    if (s ~= nil) then
			wflist[#wflist+1] = {
				Channel = 0,
				SSID    = '',
				Signal  = 0,
				WPA     = 0,
				WPA2    = 0,
				PSK     = 0,
				PSK2    = 0,
				TKIP		= 0
			}
    end
		s,e = string.find(i, 'Channel:[%d]+')
		if (s ~= nil) then
		   -- print(string.sub(i, s + 8, e))
       wflist[#wflist]['Channel'] = string.sub(i, s+8, e)
    end

    s,e = string.find(i, 'ESSID:"[%S]+"')
    if (s ~= nil) then
       -- print(string.sub(i, s + 7, e-1))
       wflist[#wflist]['SSID'] = string.sub(i, s+7, e-1)
    end

    s,e = string.find(i, 'Signal level=[-%d]+')
    if (s ~= nil) then
       -- print(string.sub(i, s + 14, e) .. ' dBm')
       wflist[#wflist]['Signal'] = string.sub(i, s+14, e)
    end

    s,e = string.find(i, 'IE:.+WPA2 Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA2'] = 1
    end

    s,e = string.find(i, 'IE:.+WPA Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK2')
    if (s ~= nil) then
       wflist[#wflist]['PSK2'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK')
    if (s ~= nil) then
       wflist[#wflist]['PSK'] = 1
    end
	
		s,e = string.find(i, 'CCMP TKIP')
		if (s ~= nil) then
			wflist[#wflist]['TKIP'] = 1
		end
	end

	return 0, wflist
end

function get_wifi_list()
	--[[
	s = luci.sys.exec('iwpriv ra0 set SiteSurvey=0; iwpriv ra0 get_site_survey | awk \'{ if (NR > 2 && NF == 9) print "{ wifiSsid = "$2", wifiRssi = "$5", wifiAuth = "$4" }" }\'')
	my_print(s)
	return 0, json.encode(s)
	-- iwlist  wlan1 scanning | grep "Channel:\|ESSID:\|Signal\|Group Chpher\|Pairwise Ciphers\|Authentication Suites\|WPA" | sed "s/:/ /g" | sed "s/level=//g"| awk '{if (NR > 1 && $1 == "Channel") print ","; if($1==
"Channel") print "{chan = "$2","; else if ($2 == "Signal") print "rssi = "$3","; else if($1 == "ESSID") print "ssid = "$2",auth=AUTH_"; else if (match($2,"WPA")) print "WPA"; else if(match($3,"WPA2")) print "WPA2"; else if(
$1 == "Authentication") print $4"}" }'
	--]]
	
	-- v0.0.2
	--[[
	return 0, {{
			wifiSsid	= 'network1',
			wifiRssi	= -5,	
			wifiAuth	= 3
	},{
			wifiSsid	= 'network2',
			wifiRssi	= -10,	
			wifiAuth	= 3
	}}
	--]]

	local wfenable = uci:get('wireless', 'radio0', 'disabled')
	if (wfenable == '1') then
		uci:set('wireless', 'radio0', 'disabled', '0')
		uci:commit('wireless')
		os.execute('wifi; sleep 8')
	end

	-- ps=luci.util.execi('iw dev wlan1 scan')
	local ps=luci.util.execi('iwlist wlan1 scanning')
	if not ps then
        return 0, {}
	else
        ps()
	end

	local wflist = {}
	for i in ps do
		local s,e = string.find(i, 'Cell.+Address:+')
    if (s ~= nil) then
			wflist[#wflist+1] = {
				Channel = 0,
				SSID    = '',
				Signal  = 0,
				WPA     = 0,
				WPA2    = 0,
				PSK     = 0,
				PSK2    = 0
			}
    end
		s,e = string.find(i, 'Channel:[%d]+')
		if (s ~= nil) then
		   -- print(string.sub(i, s + 8, e))
       wflist[#wflist]['Channel'] = string.sub(i, s+8, e)
    end

    s,e = string.find(i, 'ESSID:"[%S]+"')
    if (s ~= nil) then
       -- print(string.sub(i, s + 7, e-1))
       wflist[#wflist]['SSID'] = string.sub(i, s+7, e-1)
    end

    s,e = string.find(i, 'Signal level=[-%d]+')
    if (s ~= nil) then
       -- print(string.sub(i, s + 14, e) .. ' dBm')
       wflist[#wflist]['Signal'] = string.sub(i, s+14, e)
    end

    s,e = string.find(i, 'IE:.+WPA2 Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA2'] = 1
    end

    s,e = string.find(i, 'IE:.+WPA Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK2')
    if (s ~= nil) then
       wflist[#wflist]['PSK2'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK')
    if (s ~= nil) then
       wflist[#wflist]['PSK'] = 1
    end
	end


	-- luci.util.dumptable(wflist)
	local wl = {}
	for i=1,#wflist do
		-- print('-------------')
		-- luci.util.dumptable(wflist[i])
		if (wflist[i]['SSID'] ~= '') then
			wl[#wl+1] = {}

			wl[#wl]['ssid'] = wflist[i]['SSID']
			wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['Signal'])
			wl[#wl]['chan'] = tonumber(wflist[i]['Channel'])

			-- WPA WPA2 PSK PSK2
			local auth = 'AUTH'
			if ( wflist[i]['WPA'] == 1 and wflist[i]['WPA2'] == 1 ) then
				auth = auth .. '_WPA_WPA2'
			elseif (wflist[i]['WPA'] == 1) then
				auth = auth .. '_WPA'
			elseif (wflist[i]['WPA2'] == 1) then
				auth = auth .. '_WPA2'
			else
				auth = auth .. '_NONE'
			end

			if (auth ~= 'AUTH_NONE') then
				if (wflist[i]['PSK'] == 1) then
					auth = auth .. '_PSK'
				elseif (wflist[i]['PSK2'] == 1) then
					auth = auth .. '_PSK2'
				end
			end

			local auth_int = 0
			if (auth == 'AUTH_NONE') then
				auth_int = 0
			elseif (auth == 'AUTH_WPA_PSK') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2_PSK') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2_PSK') then
				auth_int = 4
			elseif (auth == 'AUTH_WPA_PSK2') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2_PSK2') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2_PSK2') then
				auth_int = 4
			elseif (auth == 'AUTH_WPA') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2') then
				auth_int = 4
			else 
				auth_int = 5
			end

			wl[#wl]['auth'] = auth_int
		end
	end

	-- luci.util.dumptable(wl)
	
	return 0, wl
	
	-- v0.0.4
	-- auth - 0->auth_open, 1->auth_wep, 2->auth_wap_psk, 3->auth_wpa2_psk, 4->auth_wpa_wpa2_psk, 5->auth_max
	--[[
	return 0, {{
		ssid			= 'network1',
		rssi			= -5,
		auth			= 0,
		chan			= 11
	}, {
		ssid			= 'network1',
		rssi			= -10,
		auth			= 0,
		chan			= 12
	}}
	--]]
end

--: cmd handler ----------------------------------------------------------------------------
function cmd_create_entry_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= yale_lock_create_entry_code(data, traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end

function cmd_alter_entry_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= yale_lock_alter_entry_code(data, traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end

function cmd_set_entry_code_state(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd,  code		= yale_lock_set_entry_code_state(data,traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end


function cmd_set_lock_state(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= yale_lock_set_state(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	--[[
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	--]]
	return ucmd
end

function cmd_set_auto_lock_time_insec(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= yale_lock_set_auto_time_insec(data, traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end

function cmd_clr_entry_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= yale_lock_clr_entry_code(data, traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end

function cmd_get_one_touch_lock(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
		--backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
		local ucmd, code = yale_lock_get_one_touch_lock(data, traceId);
		return ucmd
end

function cmd_set_one_touch_lock(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code = yale_lock_set_one_touch_lock(data, traceId);
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return ucmd
end

--[[
function cmd_lock_keypad_use_notify(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code		= yale_lock_time_insec(data, traceId)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end
--]]

function cmd_get_debug_info(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code		= set_device_config_mode(data)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
				debugInfo = json.encode({
					rssip						= get_gateway_rsrp(),
					snr							= get_gateway_snr(),
					qccid						= get_gateway_qccid(),
					uptime					= get_gateway_uptime(),
					staip						= get_gateway_sta_ip(),
					dialcnt					= get_gateway_dial_cnt(),
					mqttoffcnt			= get_gateway_off_cnt(),
					qnwinfo					= get_gateway_qnwinfo(),
					qnwchan					= get_gateway_qnwchan(),
					wifi_ip					= get_gateway_sta_ip()
				})
			}
		}
	}
	local sret = json.encode(ret['payload'])

	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end

function cmd_get_subdevice_signal(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
		--backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
		return get_subdevice_signal(data, traceId);
end


function cmd_device_config_mode(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code		= set_device_config_mode(data)
	local message	= 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end
function cmd_device_upgrade(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local subDeviceId = data['subDeviceId']
	if (subDeviceId ~= nil) then
		local code = upgrade_device(data, traceId)
	else
		local code = upgrade_gateway(data)
		backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)

		local message = 'OK'
		local ret=  {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
				}
			}
		}
		local sret = json.encode(ret['payload'])
		my_print('sret is ' .. sret)
		mqtt_publish('response', sret, #sret)
	end

	return rspnull()
end
function cmd_gateway_upgrade(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code = upgrade_gateway_firmware(data, traceId)
	local message = 'OK'
	local ret=  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end

function cmd_batch_delete_sub_device(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code = batch_delete_sub_device(data)
	local message = 'OK'
	local ret  = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end
function cmd_add_sub_device(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	-- subDeviceId = data['subDeviceId']
	-- backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, subDeviceId, deviceModel, data)
	return add_sub_device(traceId, data)

	--[[
	local code,subdevice = add_sub_device(traceId, data)
	local message = ''
	return  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= subdevice
		}
	}
	--]]
end
function cmd_reset_device(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code,resetType = reset_gateway(data, traceId)
	local message = 'OK'
	local ret =   {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
				resetType = resetType,
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end

function cmd_set_device_arm_mode(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code = set_arm_mode(data)
	local message = 'OK'
	local ret =  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is ' .. sret)
	mqtt_publish('response', sret, #sret)
	return rspnull()
end
function cmd_get_device_info(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	if (false and fs.access('/tmp/.zb_devicelist')) then
		local subDeviceList = {}
		local svalue = fs.readfile('/tmp/.zb_devicelist')
		local value = json.decode(svalue)
		local device_list=value['device_list']
		for i=1,#device_list do
			local ji=device_list[i]

			--
			-- my_print(ji)
			-- {"mac":"000d6f000be63f63","type":"1218","subtype":"","ModelStr":"SmartPlug","battery":100,"online":1,"version":"1.2",
			--	"model":"Dusun","epList":[{"ep":"01","type":"1218"}]}
			
			local _mac				= ji.mac
			local type			= ji.type
			local subtype		= ji.subtype
			local modelstr	= ji.ModelStr
			modelstr	= devstr_search_by_modelstr(modelstr)
			local battery		= ji.battery
			local online		= ji.online
			local version		= ji.version
			local model			= ji.model
			local state			= ji.state or '0'
			local mcuver		= ji.mcuversion == '' and nil or ji.mcuversion
			local zigver		= ji.zigversion == '' and nil or ji.zigversion
			if (modelstr == 'ask110') then
				state = nil
			end
			local tamper		= 0
			local autoLockTimeInSec = ji.autoLockTimeInSec
		
			if (modelstr ~= 'UnknownDevice') then
					subDeviceList[#subDeviceList+1] = {
					subDeviceId =	_mac,
					deviceModel	= modelstr,
					firmVersion = version,
					firmInfo		= mcuver == nil and nil or {
						{
							firmName		= 'mcu',
							firmVersion	= mcuver
						},
						{
							firmName		= 'zigbee',
							firmVersion	= zigver
						}
					},
					sn					= _mac,
					online			= online,
					state				= tonumber(state),
					battery			= battery,
					autoLockTimeInSec = autoLockTimeInSec
				}
			end
			--voltage			= battery
			--tamper			= tamper,
		end

		my_print(json.encode(subDeviceList))

		local message = 'OK'
		local code = 0

		local dgstr			= dginfo('a')
		local dgjson		= json.decode(dgstr)

		local nrssi			= dgjson['csq']
		my_print('nrssi:' .. nrssi)
		nrssi = luci.util.exec('echo -n' .. nrssi .. ' | cut -d "," -f 1 | cut -d ":" -f 2')
		local ntype = get_gateway_network_type()
		if (ntype == '4G') then
			if (nrssi == '') then
				nrssi = 0
			else
				nrssi = tonumber(nrssi)
			end
		else
			nrssi = luci.util.exec('iwpriv ra0 stat | grep RSSI | xargs | cut -d " " -f 3')
			nrssi = tonumber(nrssi)
		end
		
		local nquality	= dgjson['qcsq']
		my_print('nquality:' .. nquality)
		nquality = luci.util.exec('echo ' .. nquality .. ' | cut -d "," -f 4')
		nquality = string.gsub(nquality, "%s+", "")
		if (#nquality == 0) then
			nquality = nil
		else
			nquality = tonumber(nquality)
		end

		local nact			= dgjson['qnwinfo']
		my_print('nact:' .. nact)
		nact = luci.util.exec('echo ' .. nact .. ' | cut -d "," -f 1 | cut -d ":" -f 2')
		nact = string.gsub(nact,"%s+", "")
		if (#nact == 0) then
			nact = nil
		end

		local nband			= dgjson['qnwinfo']
		my_print('nband:' .. nband)
		nband = luci.util.exec('echo ' .. nband .. ' | cut -d "," -f 3')	
		nband = string.gsub(nband,"%s+", "")
		if (#nband == 0) then
			nband = nil
		end

		local nid				= dgjson['qspn']
		my_print('nid :' .. nid)
		nid = luci.util.exec('echo ' .. nid .. '  | cut -d "," -f 1 | cut -d ":" -f 2')
		nid = string.gsub(nid, "%s+", "")
		if (#nid == 0) then
			nid = nil
		end

		--my_print('nrssi : ' .. nrssi .. ', nquality:' .. nquality .. ',nact:' .. nact .. 'nband:' .. nband)

		local ret = {
			fixedHeader			= fixedHeader,
			variableHeader	= variableHeader,
			payload					= {
				traceId		= traceId,
				code			= code,	
				msg				= message,
				result		= {
					deviceId				= get_gateway_device_id(),			
					deviceModel			= get_gateway_model(),
					firmVersion			= get_gateway_version(),
					subDeviceNumber	= #device_list,
					battery					= get_gateway_battery(),
					powerType				= get_gateway_power_type(),
					armMode					= get_gateway_arm_mode(),
					sn							= get_gateway_sn(),
					networkType			= get_gateway_network_type(),
					networkId				= get_gateway_network_id(),
					networkRssi		  = nrssi,
					subDeviceList		= subDeviceList
				}
			}
		}
		--networkQuality	= nquality,
		--networkAct			= nact,
		--networkBand			= nband,

		if (nact ~= nil) then
			ret['payload']['result']['networkAct'] = nact
		end
		if (nband ~= nil) then
			ret['payload']['result']['networkBand'] = nband
		end
		if (nquality ~= nil) then
			ret['payload']['result']['networkQuality'] = nquality
		end
		if (nid ~= nil) then
			ret['payload']['result']['networkId'] = nid
		end

		--networkStrength = get_gateway_network_strength(),
		local sret = json.encode(ret['payload'])
		sret = string.gsub(sret, '{}', '[]')
		my_print('sret is' .. sret)
		mqtt_publish('response', sret, #sret)
		return get_sub_device_list(traceId)
	else
		backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
		return get_sub_device_list(traceId)
	end
end
function remote_config_wifi_func(data) 
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		set_wifi_mtksdk(data, 'traceId');
	else
		set_wifi(data, 'traceId')
	end
end
function cmd_set_wifi(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local code = 0
	local message		= 'OK'
	local ret =  {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
				connectState = code
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	local configId	= data['configId'] or "noconfigId"
	local ret = {
		context = {
			traceId					= get_uuid(),
			requestTime			= time_ms(),
			requestMethod		= 'configStateNotify',
			deviceId				= get_gateway_device_id(),
			deviceModel			= get_gateway_model()
		},
		data = {
			configId				= configId,
			step						= 'RECEIVE_CONFIG',
			code						= 0, 
			msg							= 'OK',
			debugInfo				= {}
		}
	}
	sret = json.encode(ret)
	my_print('sret is' .. sret)
	mqtt_publish('report', sret, #sret)


	cache_schedule_add('RemoteConfigWifi', 4, 'remote_config_wifi_func', data)
	return rspnull()
end
function cmd_get_wifi_list(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	my_print('cmd_get_wifi_list')
	local message		= 'OK'

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	local code = 0
	local wifiList = {}
	if (mtksdk == '1') then
		code, wifiList	= get_wifi_list_mtksdk()
	else
		code, wifiList	= get_wifi_list()
	end

	my_print('code is ' .. code .. ',wifilist is ' .. json.encode(wifiList))
	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= message,
			result		= {
				wifiList = wifiList
			}
		}
	}
	local sret = json.encode(ret['payload'])
	sret = string.gsub(sret, '{}', '[]')
	mqtt_publish('response', sret, #sret)
	return rspnull()
end


function cmd_get_remote_shell(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	luci.util.exec('remote_shell.sh&')

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= 0,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	mqtt_publish('response', sret, #sret)
	return rspnull()
end

function cmd_set_keypad_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= keypad_set_code(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_del_keypad_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= keypad_del_code(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_get_keypad_code(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= keypad_get_code(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_get_keypad_info(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= keypad_get_info(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_set_change_mode_delaytime(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= keypad_set_arm_delaytime(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	mqtt_publish('response', sret, #sret)
	return ucmd;
end

-- siren
function cmd_get_siren_effected_devices(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_get_effected_devices(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_update_siren_effected_devices(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_update_siren_effected_devices(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	return ucmd
end

function cmd_set_siren_volume(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_set_volume(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	return ucmd
end
function cmd_get_siren_volume(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_get_volume(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_set_siren_sounding(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_set_siren_sounding(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}

	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	return ucmd
end
function cmd_get_siren_auto_off(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_get_auto_off(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_set_siren_auto_off(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= siren_set_auto_off(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	return ucmd
end


-- repeater
function cmd_set_repeater_switch(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_set_switch(data, traceId)

	local ret = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			traceId		= traceId,
			code			= code,	
			msg				= 'OK',
			result		= {
			}
		}
	}
	local sret = json.encode(ret['payload'])
	my_print('sret is' .. sret)
	mqtt_publish('response', sret, #sret)

	return ucmd
end

function cmd_get_repeater_schedule(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_get_schedule(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_del_repeater_schedule(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_del_schedule(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_add_repeater_schedule(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_add_schedule(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_upt_repeater_schedule(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_mod_schedule(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_get_repeater_timer(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_get_timer(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_add_repeater_timer(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_add_timer(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_del_repeater_timer(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_del_timer(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_upt_repeater_timer_state(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_upt_timer(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end
function cmd_get_repeater_connectted_devices(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	local ucmd, code		= repeater_get_connectted_devices(data, traceId)
	backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
	return ucmd
end

local cmd_handlers = {
	hub_ds					= { 
		getWifiList			= cmd_get_wifi_list,
		setDeviceWifi		= cmd_set_wifi,
		getDeviceInfo		= cmd_get_device_info,
		setDeviceArmMode= cmd_set_device_arm_mode,
		resetDevice			= cmd_reset_device,
		addSubDevice		= cmd_add_sub_device,
		batchDeleteSubDevice = cmd_batch_delete_sub_device,
		deviceUpgrade		= cmd_device_upgrade,
		gatewayUpgrade	= cmd_gateway_upgrade,
		deviceConfigMode= cmd_device_config_mode,
		getDebugInfo		= cmd_get_debug_info,

		getSubDeviceSignal = cmd_get_subdevice_signal,

		createEntryCode				= cmd_create_entry_code,
		alterEntryCode				= cmd_alter_entry_code,
		setEntryCodeState			= cmd_set_entry_code_state,
		setLockState					= cmd_set_lock_state,
		setAutoLockTimeInSec	= cmd_set_auto_lock_time_insec,
		clearEntryCode				= cmd_clr_entry_code,

		getOneTouchLock			  = cmd_get_one_touch_lock,
		setOneTouchLock				= cmd_set_one_touch_lock,

		remoteShell						= cmd_remote_shell,

		setKeypadCode					= cmd_set_keypad_code,
		deleteKeypadCode			= cmd_del_keypad_code,
		getKeypadCode					= cmd_get_keypad_code,
		getKeypadInfo					= cmd_get_keypad_info,
		setChangeModeDelayTime= cmd_set_change_mode_delaytime,

		getSirenEffectedDevices			= cmd_get_siren_effected_devices,
		updateSirenEffectedDevices	= cmd_update_siren_effected_devices,
		setSirenVolume							= cmd_set_siren_volume,
		getSirenVolume							= cmd_get_siren_volume,
		setSirenSounding						= cmd_set_siren_sounding,
		getSirenAutoOff							= cmd_get_siren_auto_off,
		setSirenAutoOff							= cmd_set_siren_auto_off,

		setRepeaterSwitch						= cmd_set_repeater_switch,
		getRepeaterSchedule					= cmd_get_repeater_schedule,
		delRepeaterSchedule					= cmd_del_repeater_schedule,
		addRepeaterSchedule					= cmd_add_repeater_schedule,
		updateRepeaterSchedule			= cmd_upt_repeater_schedule,
		getRepeaterTimer						= cmd_get_repeater_timer,
		addRepeaterTimer						= cmd_add_repeater_timer,
		delRepeaterTimer						= cmd_del_repeater_timer,
		updateRepeaterTimerState		= cmd_upt_repeater_timer_state,
		getRepeaterConnectedDevices	= cmd_get_repeater_connectted_devices,
	}
}
-- lockKeypadUseNotify		= cmd_lock_keypad_use_notify
function cmd_handler_search(deviceModel, requestMethod) 
	cmd_handler_map = cmd_handlers[deviceModel]
	if (cmd_handler_map == nil) then
		my_print('not support device model : ' .. deviceModel)
		return nil
	end

	command_handler = cmd_handler_map[requestMethod]
	if (command_handler == nil) then
		my_print('not support method:' .. requestMethod)
		return nil
	end

	return command_handler
end

--: Hanlder For Msg And Cmd------------------------------------------------------------------
function valid_fixedHeader(fixedHeader) 
	return true
end
function valid_variableHeader(variableHeader) 
	return true
end
function valid_payload(payload) 
	return true
end
function handler_msg(jmsg) 
	local type = jmsg['type']
	if (type == nil) then
		my_print('no type')
		return rspnull()
	end

	local from	= jmsg['from']
	if (from == nil) then
		my_print('no from')
		return rspnull()
	end

	local data = jmsg['data']
	if (data == nil) then
		my_print('no type')
		return rspnull()
	end
	
	local attr	= data['attribute'] or ''
	local mac		= data['mac'] or ''
	local id		= data['id'] or ''
	local code	= data['code'] or ''
	local value = data['value'] or ''

	my_print('type:[' .. type .. '],from:[' .. from .. '],attr:[' .. attr .. '],mac:[' .. mac .. ']\n')
	local rpt_handler = rpt_handler_search(type, from, attr, mac)
	if (rpt_handler == nil) then
		my_print('not support msg!')
		return rspnull()
	end

	my_print('execute rpt ' .. type .. ' handler')
	return rpt_handler(type, from, attr, mac, id, code, value)
end

function handler_cmd(_jcmd) 
	local jcmd = { fixedHeader = {}, variableHeader = {}, payload = _jcmd}

	local fixedHeader = jcmd['fixedHeader']
	if (fixedHeader == nil) then
		my_print('no fixedHeader')
		return rspnull()
	end
	if (valid_fixedHeader(fixedHeader) ~= true) then
		my_print('error fixedHeader')
		return rspnull()
	end
	
	local variableHeader = jcmd['variableHeader']
	if (variableHeader == nil) then
		my_print('no variableHeader')
		return rspnil()
	end
	if (valid_variableHeader(variableHeader) ~= true) then
		my_print('error variableHeader')
		return rspnull()
	end

	
	local payload = jcmd['payload']
	if (payload == nil) then
		my_print('no payload')
		return rspnull()
	end

	if (valid_payload(payload) ~= true) then
		my_print('error payload')
		return rspnull()
	end

	
	local context				= payload['context']
	local traceId				= context['traceId']
	local requestMethod = context['requestMethod']
	local requestTime		= context['requestTime']
	local deviceId			= context['deviceId']
	local deviceModel		= context['deviceModel']

	local data					= payload['data']

	if (data ~= nil and data['subDeviceId'] ~= nil) then
			data['subDeviceId'] = string.lower(data['subDeviceId'])
	end

	if (deviceId ~= get_gateway_device_id()) then
		my_print('not correct deviceId:' .. deviceId)
		return rspnull()
	end
	
	my_print('requestMethod:[' .. requestMethod .. '],deviceId:[' .. deviceId .. '],deviceModel:[' .. deviceModel .. ']\n')

	cmd_handler=cmd_handler_search(deviceModel, requestMethod)
	if (cmd_handler == nil) then
		my_print('not support cmd:' .. requestMethod .. ' for deviceModel:' .. deviceModel .. '\n')
		return rspnull()
	end

	my_print('cmd handler for ' .. deviceId)
	return cmd_handler(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data)
end

--: Call From Exe--------------------------------------------------------------------------------
function msgin(msg) 
	cache_load()

	local jmsg = json.decode(msg)
	
	my_print('jmsg is' .. msg)
	
	local jret, topic = handler_msg(jmsg)
	
	local sret = json.encode(jret)

	if ( #sret > 6 ) then
		if (topic == nil) then
			topic = 'report'
		end

		my_print('publish to ' .. topic .. ': ' .. sret)

		mqtt_publish(topic, sret, #sret)
	end
	
	cache_save()

	return '{}'
end

function cmdin(cmd) 
	cache_load()

	--cmd = string.gsub(cmd, '\\', '\\\\')

	local jcmd = json.decode(cmd)
	
	my_print('jcmd is' .. cmd)
	
	local jret = handler_cmd(jcmd)

	local sret = json.encode(jret)
	my_print('sret is :' .. sret)

	--topic = 'report'
	--mqtt_publish(topic, sret, #sret)
		
	cache_save()
	return sret
end

function callf(fname) 
	cache_load()

	if (type(fname) ~= "string")  then
		return json.encode(rspnull())
	end
	if (type(_G) ~= "table") then
		return json.encode(rspnull())
	end
	local f = _G[fname]

	--[[
	for v in fname:gmatch("[^%.]+") do
		if (type(f) ~= "table") then
			return json.encode(rspnull())
		end
		f=f[v]
	end
	--]]

	if (type(f) ~= "function") then
		return json.encode(rspnull())
	end

	
	local s=json.encode(f())
	-- my_print(s)
	
	cache_save()
	return s

end

-- [dn cmd format]:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"","requestTime":1576505177776,"requestMethod":"","deviceId":"","deviceModel":""},"data":{}}}
-- [up rsp format]:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"response","packetId":-1},"payload":{"traceId":"","code":0,"msg":"","result":{}}}
-- [up rpt format]:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":1,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"report","packetId":"16"},"payload":{"context":{"traceId":"","requestTime":1576505177776,"requestMethod":"","deviceId":"","deviceModel":""},"data":{}}}

-- *********************************************************!!!!!!!!!!!!!!
-- Please Replace the DeviceId to the correct deviceId : dsb66d9c8e656edcaa1eb6776a5b923a5a
-- Please Replace the subDeviceid to the correct subDeviceId : use mac

-- Command && rsp payload:
-- Get Wifi List:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"getWifiList","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{"wifiList":[{"wifiSsid":"network1","wifiRssi":-5,"wifiAuth":3},{"wifiSsid":"network2","wifiRssi":-10,"wifiAuth":3}]}}

-- Setting Wifi:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setDeviceWifi","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"wifiSsid":"network1","wifiPwd":"12345678"}}}
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setDeviceWifi","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"ssid":"AAAAAA","pwd":"dl123456", "chan":"6"}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{"connectState":0}}

-- Get Gateway Info:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"getDeviceInfo","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{"deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity","firmVersion":"1.0.15","subDeviceNumber":2,"battery":50,"powerType":0,"armMode":1,"sn":"1235464213482","networkType":"4G","networkStrength":-15,"networkId":"LTE","subDeviceList":[{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3","deviceModel":"etekcity","sn":"5465454654646","firmVersion":"1.0.1","online":1,"state":1,"tamper":0,"voltage":0},{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3","deviceModel":"etekcity","sn":"5465454654646","firmVersion":"1.1.15","online":1,"state":0,"tamper":0,"voltage":0}]}}

-- Set Arm Mode:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setDeviceArmMode","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"armMode":1}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}


-- Please Replace the DeviceId to the correct deviceId : dsb66d9c8e656edcaa1eb6776a5b923a5a
-- Reset Gateway:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"resetDevice","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"resetType":80}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}

-- Add SubDevice
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"addSubDevice","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3"}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3","deviceModel":"etekcity","sn":"5465454654646","firmVersion":"1.0.1","online":1,"state":1,"tamper":0,"voltage":0}}

-- Batch Delete SubDevice:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"batchDeleteSubDevice","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"subDeviceIdList":["01c6e408d4232a422882c2919e390f55b3"]}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}

-- Upgrade Gateway:
-- {"fixedHeader":{"messageType":"PUBLISH","isDup":false,"qosLevel":0,"isRetain":false,"remainingLength":"xx"},"variableHeader":{"topicName":"request","packetId":-1},"payload":{"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"deviceUpgrade","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"DSI0134"},"data":{"upgradeUrl":"http://****************","firmVersion":"1.0.20"}}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}


-- Rpt Payload:
-- connect success rpt:
-- firmVersion string[yes]
-- subDeviceNumber int[yes]
-- battery int[yes]
-- powerType int[yes]
-- armMode int[yes]
-- sn string[yes]
-- networkType string[yes]
-- networkStrength int[yes]
-- networkId string[yes] wifi ssid or 4g isp
-- subDeviceList list
-- subDeviceId string[yes]
-- deviceModel string[yes]
-- sn string[yes]
-- firmVersion string[yes]
-- online int[yes]
-- state int[no]
-- tamper int[no]
-- voltage int[no]
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"deviceConnectNotify","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity"},"data":{"deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity","firmVersion":"1.0.15","subDeviceNumber":2,"battery":50,"powerType":0,"armMode":1,"sn":"1235464213482","networkType":"4G","networkStrength":-15,"networkId":"LTE","subDeviceList":[{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3","deviceModel":"etekcity","sn":"5465454654646","firmVersion":"1.0.1","online":1,"state":1,"tamper":0,"voltage":0},{"subDeviceId":"01598e8fbc92224a809ad6ca5b19ce2b6c","deviceModel":"etekcity","sn":"5465454654646","firmVersion":"1.1.15","online":1,"state":0,"tamper":0,"voltage":0}]}}

-- gateway status change rpt:
-- networkType string[no] 4G, wifi
-- networkId string[no] wifi ssid or 4g isp
-- networkRssi int[yes]
-- powerType int[no] 0->battery, 1->power outlet
-- armMode int[no] 0->disarm, 1->arm home, 2->arm away
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"deviceStateChangeNotify","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity"},"data":{"networkType":"wifi","networkId":"network3"}}


-- gateway timer rpt:
-- battery int[yes]
-- networkType string[yes] 4G, wifi
-- networkRssi int[yes] 
-- networkId string[yes] wifi ssid or 4g isp
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"deviceRegularNotify","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity"},"data":{"battery":50,"networkRssi":-20,"networkType":"wifi","networkId":"network3"}}


-- gateway upgrade rpt:
-- upgradeState string[yes], 1-100->download progress, 255->upgrade failed, 207->download complete, 208->checksum ok,to upgrade
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"upgradeNotify","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity"},"data":{"upgradeState":"80"}}

-- device status rpt:
-- subDeviceid string[yes]
-- online int[no]
-- state  int[no]
-- tamper int[no]
-- voltage int[no]
-- firmVersion string[no]
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"subDeviceStateChangeNotify","deviceId":"dsa9143909c31b460f9739c59a697afe60","deviceModel":"etekcity"},"data":{"subDeviceId":"01c6e408d4232a422882c2919e390f55b3","state":0}}


-- yale lock:
-- createEntryCode:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"createEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"1","entryCode":"1234","codeType":2}} // unrestrict
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"createEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"0","entryCode":"123457","codeType":1}} // admin
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"createEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"1","entryCode":"1234","codeType":3, "startDateTime":1583732100, "endDateTime":1615268100}} // tmp, year day
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"createEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"1","entryCode":"1234","codeType":4, "startTime":"12:00", "endTime":"15:00", "repeats":127}} // repeat, week day
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}
--
-- alterEntryCode:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"alterEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"1","entryCode":"4567","codeType":2}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}
--
-- setEntryCodeState:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setEntryCodeState","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","entryCodeId":"1","entryCodeState":1}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}
--
-- clearEntryCode
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"clearEntryCode","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0"}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}}
--
-- setLockState:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setLockState","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","lockState":1}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}} 
--
-- setAutoLockTimeInSec:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"setAutoLockTimeInSec","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","autoLockTimeInSec":30}}
-- {"traceId":"a5ca334d1049499e9880c2f9bb7becb8","code":0,"msg":"","result":{}} 
--
-- lockKeypadUseNotify:
-- {"context":{"traceId":"a5ca334d1049499e9880c2f9bb7becb8","requestTime":1576505177776,"requestMethod":"lockUseNotify","deviceId":"ds5950e2a5f8a30c378fdc10079332b1e6","deviceModel":"hub_ds"},"data":{"subDeviceId":"000d6f00108de6a0","deviceModel":"smart_lock_ds","eventType":1,"eventResult":1,"eventTime":1576505177776,"eventMsg":{"entryCodeId":"1"}}}

-- TODO:
-- 1. getwifi list implement and ret value ok
-- 2. subdevice state, tamper, tamper no ok
-- 3. setwifi implement and ret value ok
-- 5. upgrade download ok, start upgrade, md5 ok message add, upgrade argments' need add md5sum  ok
-- 4. configMode ok
-- 5. 'deviceRegularNotify', modify gateway.status to device to record a flag count, and  device_list  will be a regular notify if has the flag ok
-- 7. 'deviceStateChangeNotify',trigger message, a new ubus message such as 'gateway.state.change' when armMode, network, powerType change ok
-- 8. gateway network type and rssi, network(wifi, 4g, eth) switch state and rssi ok
-- 9. reset gateway implement ok
-- a. configStateNotify: RECEIVE_CONFIG, CONNECT_WIFI, GET_IP, DNS_RESOLVE, CONNECT_SERVER 90, wait test and check ok
-- b. add subDevice failed ret, parse the cmdResult and fetch addSubDevice command to match and send result 90%, wait test and check


-- c. deviceConnectNotify data remove devviceModel and deviceId ok
-- d. deviceStateChangeNotify deviceId, deviceModel  
-- d. configStateNotify DSN_RESERVER add routerRssi ok
-- e. RegularNotify too more segment, only devcieId and battery ok


-- x. 6.6->ota ok wait test green, yellow on shot, green blink, on ok,
-- x. 6.3 reset->green(pwrled), yellow(zigbee) on for 1second, then off, green blink 1, green on	 
-- x. 6.53->zigbee led 500ms -> 1s, don't do this, hold the old 500ms 500ms ok
-- x. apn config wifi should set to ap ok
-- x. 6.52,6.51, 6.4-> apn config wifi red led 20 ~60s, 20 return and success off, else blink for 60s ok, modify to 2*60, because 60 too short
-- x. 6.2 bootup, green(pwrled), yellow(zigbee) on for 1second, green blink 1, green on ok
-- x. when wifi config no or failed, restore to old if exsit
-- x. the router add metric setting /lib/netifd/dhpc.script && netmge.sh
-- x. chenbei lock time sync, time see cli.c timeset


-- 1. bufang, sensor rpt, keypad remove alarm, 
-- 2. chefang, door, leak rpt, pir rpt in 5 min no motion
-- 3. bufang/arm.db fild swap ok
-- 4. ERROR: The RSN IE of this received Probe-resp, optimize

--cmd_get_subdevice_signal(nil, nil, '1111111111111', 'getSubDeviceSignal',time_ms(), 'dsdf2fef92859173d87e6c1017928ef226', 'hub_ds', {subDeviceId = '84fd27fffe84d084'})

--upgrade_device({subDeviceId = '84fd27fffe84d084', subDeviceModel = 'asl120', upgradeUrl = 'http://114.215.195.44:8080/au/X', md5sum = '111', firmVersion = '1.1.1',firmInfo = {firmName = 'mcu', firmVersion = '1.0.0'}}, 'traceId11111111') 
